    SUBROUTINE KFPARA(NCUYES,ICUYES,J,LSB,HTOP,HBOT &
    ,                     CNVTOP,CNVBOT,NSHALL)
!     ******************************************************************
!$$$  SUBPROGRAM DOCUMENTATION BLOCK
!                .      .    .
! SUBPROGRAM:    KFPARA      CONVECTIVE PRECIPITATION PARAMETERIZATION
!   PRGRMMR: KAIN            ORG: W/NP2      DATE: 00-04-13

! ABSTRACT:
!     KFPARA IS THE CENTRAL SUBROUTINE IN THE K-F PARAMETERIZATION.
!     SHALLOW CONVECTION IS INCLUDED WITHOUT CAPE DEPENDENCE.


! PROGRAM HISTORY LOG:
!   ??-??-??  KAIN       - ORIGINATOR
!   00-04-13  BLACK      - INCORPORATED INTO THE ETA MODEL

! USAGE: CALL KFPARA FROM SUBROUTINE KFDRIVE

!   INPUT ARGUMENT LIST:
!     NCUYES - NUMBER OF GRID POINTS TO CHECK ON THIS J-SLICE
!     ICUYES - ARRAY CONTAINING THE I-VALUES FOR GRID POINTS TO CHECK
!     J      - THE ROW NUMBER
!     LSB    - ARRAY CONTAINING l VALUES FOR EACH NCUYES POINT AT WHICH
!              CHECKS FOR CONVECTIVE INITAITION SHOULD BEGIN

!   OUTPUT ARGUMENT LIST:
!     HTOP   - ARRAY USED TO STORE L VALUE FOR CLOUD TOP (FOR RADTN)
!     HBOT   - ARRAY USED TO STORE L VALUE FOR CLOUD BOTTOM (FOR RADTN)
!     CNVTOP - ARRAY USED TO STORE L VALUE FOR CLOUD TOP (FOR CHKOUT)
!     CNVBOT - ARRAY USED TO STORE L VALUE FOR CLOUD BOTTOM (FOR CHKOUT)
!     NSHALL - COUNTER FOR NUMBER OF SHALLOW CONVECTION POINTS ACTIVATED

!   OUTPUT FILES:
!     NONE

!   SUBPROGRAMS CALLED:

!     UNIQUE:
!        CONLOAD
!        DTFRZNEW
!        ENVIRTHT
!        PROF5

!     LIBRARY:
!        NONE

!   COMMON BLOCKS: CTLBLK
!                  LOOPS
!                  MASKS
!                  VRBLS
!                  DYNAM
!                  CNVCLD
!                  PVRBLS
!                  ACMCLH
!                  KFFDBK
!                  KFLUT
!                  INDX

! ATTRIBUTES:
!   LANGUAGE: FORTRAN 90
!   MACHINE : IBM SP
!$$$
!**********************************************************************C
!                                                                      C
! ETA INPUT:   TEMPERATURE (T, K);    SPECIFIC HUMIDITY (Q, KG/KG)     C
!              PD, RES, AND PT USED TO CALCULATE PRESSURE (PASCAL)     C
!              HORIZONTAL WIND SPEED (U AND V,  M/S)                   C
!              HORIZONTAL GRID SPACING (DX,  M)                        C
!              MODEL TIME STEP (DT,  SECONDS)                          C
!              NUMBER OF TIME STEPS INTEGRATED (NTSD,  NO UNITS)       C
!              INSTANTANEOUS SURFACE SENSIBLE HEAT FLUX (TWBS, W/M^2)  C
!              INSTANTANEOUS SURFACE LATENT HEAT FLUX (QWBS, W/M^2)    C
!              ETA MODEL LEVELS (ETA,  NO UNITS)                       C
!              HALF ETA MODEL LEVELS (AETA, NO UNITS)                  C
!              MODEL LEVEL OF TOP OF PBL (PBLLEV)                      C
!              DIMENSIONS OF MODEL GRID (IMX,JMX,KMX FOR IM,JM,LM)     C
!              ARRAYS SPECIFYING WHERE H AND V ARRAYS ARE BELOW        C
!                                GROUND (LMH AND VMH,  NO UNITS)       C
!                                                                      C
!    OUTPUT:   CONVECTIVE TENDENCIES OF TEMPERATURE (DTDT), WATER      C
!              VAPOR (DQDT), CLOUD WATER (DQCDT), RAIN WATER (DQRDT)   C
!   !!!!! NOTE:  cloud water and rainwater arrays (DQCDT and DQRDT)    C
!                arrays removed for implementation in ETA model !!!!!!!C
!              AND PRECIPITATION RATE (RAINCV)                         C
!              HORIZ. LOCATION OF ACTIVE CONVECTION (NCA, NO UNITS)    C
!                                                                      C
!**********************************************************************C
!                                                                      C
!     REFERENCES:                                                      C
!                                                                      C
!         KAIN AND FRITSCH (1993):  "CONVECTIVE PARAMETERIZATION IN    C
!         MESOSCALE MODELS:  THE KAIN-FRITSCH SCHEME" IN THE REPRESEN- C
!         TATION OF CUMULUS CONVECTION IN NUMERICAL MODELS, A.M.S.     C
!         MONOGRAPH, K.A. EMANUEL AND D.J. RAYMOND, EDS., 165-170.     C
!                                                                      C
!         FRITSCH AND KAIN (1993):  "CONVECTIVE PARAMETERIZATION IN    C
!         MESOSCALE MODELS:  THE FRITSCH-CHAPPELL SCHEME" IN THE REP-  C
!         RESENTATION OF CUMULUS CONVECTION IN NUMERICAL MODELS, A.M.S.C
!         MONOGRAPH, K.A. EMANUEL AND D.J. RAYMOND, EDS., 165-170.     C
!                                                                      C
!         STENSRUD AND FRITSCH (1994), MON. WEA. REV., 2084-2104.      C
!                                                                      C
!         FRITSCH AND CHAPPELL (1980), J. ATMOS. SCI., 1722-1733.      C
!                                                                      C
! CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

!----------------------------------------------------------------------
    INCLUDE "parmeta.f90"
    INCLUDE "mpp.h"
!----------------------------------------------------------------------
    PARAMETER(LP1=LM+1,JAM=6+2*(JM-10))
    PARAMETER(KX=LM,KXP1=KX+1,ILX=IM-1,JLX=JM-1)
!----------------------------------------------------------------------
    INCLUDE "COMM_ACMCLH.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_CTLBLK.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_DYNAM.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_MASKS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_VRBLS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_PVRBLS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_LOOPS.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_CNVCLD.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_KFFDBK.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_KFLUT.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_INDX.f90"
!----------------------------------------------------------------------
    INCLUDE "COMM_MAPOT1.f90"
!----------------------------------------------------------------------
    REAL &
    HTOP  (IDIM1:IDIM2,JDIM1:JDIM2),HBOT  (IDIM1:IDIM2,JDIM1:JDIM2) &
    ,CNVTOP(IDIM1:IDIM2,JDIM1:JDIM2),CNVBOT(IDIM1:IDIM2,JDIM1:JDIM2) &
    ,QUER(KX),TKE(KX) &
    ,CLDHGT(KX),QSD(KX),DILFRC(KX),DDILFRC(KX) &
    ,THTEEG(KX),TGU(KX),QGU(KX)

    INTEGER &
    ICUYES(IDIM1:IDIM2),LSB(IDIM1:IDIM2)
!----------------------------------------------------------------------

!***  DEFINE LOCAL VARIABLES

    REAL &
    P0(KXP1),Z00(KX),T0(KX),TV0(KX),Q0(KXP1),U0(KX),V0(KX) &
    ,TU(KX),TVU(KX),QU(KX),TZ(KX),TVD(KX),QD(KX),QES(KX),THTES(KX) &
    ,TG(KX),TVG(KX),QG(KX),WU(KX),WD(KX),EMS(KX),EMSD(KX),W0(KX) &
    ,UMF(KX) ,UER(KX) ,UDR(KX) ,DMF(KX) ,DER(KX) ,DDR(KX) ,DZQ(KX) &
    ,UMF2(KX),UER2(KX),UDR2(KX),DMF2(KX),DER2(KX),DDR2(KX),DZA(KX) &
    ,THTA0(KXP1),THETEE(KX),THTAU(KX),THETEU(KX),THTAD(KX) &
    ,THETED(KX) &
    ,QLIQ(KX),QICE(KX),QLQOUT(KX),QICOUT(KX),PPTLIQ(KX),PPTICE(KX) &
    ,DETLQ(KX),DETIC(KX),DETLQ2(KX),DETIC2(KX),RATIO(KX),RATIO2(KX)

    REAL &
    DOMGDP(KX),EXN(KX),RHOE(KX),TVQU(KX),DP(KX),RH(KX) &
    ,EQFRC(KX),WSPD(KX),QDT(KX),FXM(KX),THTAG(KX),THTESG(KX) &
    ,THPA(KX),THFXIN(KX),THFXOUT(KX),QPA(KX),QFXIN(KX),QFXOUT(KX) &
    ,QLPA(KX),QLFXIN(KX),QLFXOUT(KX),QIPA(KX),QIFXIN(KX),QIFXOUT(KX) &
    ,QRPA(KX),QRFXIN(KX),QRFXOUT(KX),QSPA(KX),QSFXIN(KX),QSFXOUT(KX) &
    ,QL0(KX),QLG(KX),QI0(KX),QIG(KX),QR0(KX),QRG(KX),QS0(KX),QSG(KX)

    REAL &
    OMG(KXP1),RAINFB(KX),SNOWFB(KX)
!----------------------------------------------------------------------

!...DEFINE CONSTANTS NEEDED FOR KFPARA SUBROUTINE

    DATA P00,T00,G,CP/1.E5,273.16,9.8,1004.6/
    DATA RLF/3.339E5/
    DATA RHIC,RHBC/1.,0.90/
    DATA PIE,TTFRZ,TBFRZ,C5/3.141592654,268.16,248.16,1.0723E-3/
    DATA RATE,RV/0.03,461.5/
    DATA ALIQ,BLIQ,CLIQ,DLIQ/613.3,17.502,4780.8,32.19/
    DATA AICE,BICE,CICE,DICE/613.2,22.452,6133.0,0.61/
    DATA XLV0,XLV1,XLS0,XLS1/3.147E6,2369.,2.905E6,259.532/
!----------------------------------------------------------------------
!****************************************************************************
!----------------------------------------------------------------------

!***  OPTION TO FEED CONVECTIVELY GENERATED RAINWATER
!***  INTO GRID-RESOLVED RAINWATER (OR SNOW/GRAUPEL)
!***  FIELD.  'FBFRC' IS THE FRACTION OF AVAILABLE
!***  PRECIPITATION TO BE FED BACK (0.0 - 1.0)

    FBFRC=0.0

!----------------------------------------------------------------------
    ROVG=R/G
    GDRY=-G/CP
    DT2 =2.*DT
!----------------------------------------------------------------------

!***  SPECIFY DOWNDRAFT MASS FLUX AS FRACTION
!***  OF UPDRAFT MASS FLUX (DMFFRC)

    DMFFRC=0.9
!----------------------------------------------------------------------
!----------------------------------------------------------------------
!***
!***  LOOP OVER ALL POTENTIAL CONVECTIVE GRID POINTS
!***
!----------------------------------------------------------------------
!----------------------------------------------------------------------
    NCCNT=0

    DO 900 NC=1,NCUYES
    
    !***  VARIABLES TO ALLOW SHALLOW CONVECTION
    
        NCHM=0
        ISHALL=0
        RAD=1500.
        FBFRC=0.

    ! LG
    !--- Assume the following functional dependence for 1 - 40 km resolution:
    !       FBFRC=1. for dx=50 km, FBFRC=1. for dx=1 km, where
    
        DXR=111.*(DPHD**2+DLMD**2)**.5         ! Model resolution at equator (km)
        DXR=MIN(50., MAX(1., DXR) )
        IF (DXR <= 1.) THEN
            Fdx=0.
        ELSE
            IF ((DXR > 1.) .AND. (DXR < 50.)) THEN
                Fdx=0.6*LOG10(DXR)-0.0443162
            ELSE
                Fdx=1.
            ENDIF
        ENDIF
          FBFRC=1-Fdx
!JLG	  if (mype.eq.0)write(6,*)'FBFRC= ',FBFRC,'Fdx= ',Fdx,'DXR= ',DXR,LOG(DXR)
    !JLG  FBFRC=1.
        FBFQv=0.4   !Fraction of liquid water that evaporates
          
    ! LG
        DPMIN=5.E3
    
        I=ICUYES(NC)
        XTIME=NTSD*DT/60.
        DXSQ=DX(I,J)*DX(I,J)
        P200=PD(I,J)+PT-2.E4
        P300=PD(I,J)+PT-3.E4
        P400=PD(I,J)+PT-4.E4
        ML=0
    
    !***  DEFINE NUMBER OF LAYERS ABOVE GROUND LEVEL - CALL THIS KL
    
        KL=LMH(I,J)
        NLEV=KX-KL
        KLM=KL-1
    
    !***  INPUT A VERTICAL SOUNDING
    
    !***  THE FOLLOWING LOOP IS CRUCIAL.  THE KF SCHEME IS SET UP TO HAVE
    !***  LEVELS STARTING WITH 1 AT THE LOWEST MODEL LEVEL AND GOING UP,
    !***  WHICH IS INVERTED FROM WHAT SIGMA AND ETA COORDINATE MODELS
    !***  TEND TO USE.  THUS, THE FIRST STEP IS TO SWITCH THE ORDER OF THE
    !***  VARIABLES USED WITHIN THE KF SCHEME.  NOTE THAT THE SCHEME NEEDS
    !***  THE FOLLOWING VARIABLES
    
    !     INPUT:   TEMPERTURE (T0, K) ;    SPECIFIC HUMIDITY (Q0, KG/KG) ;
    !              HORIZONTAL WIND SPEED (U0 AND V0, M/S) ;
    !              PRESSURE (P0, PASCAL) ;  HEIGHT (Z0, M);
    !              VERTICAL MOTION (W0, M/S).
    
        DO 15 K=1,KL
            NK=KX-K+1-NLEV
            P0(K)=PD(I,J)*RES(I,J)*AETA(NK)+PT
            T0(K)=T(I,J,NK)
            Q0(K)=Q(I,J,NK)
            TKE(K)=Q2(I,J,NK)
        
        !***  SATURATION VAPOR PRESSURE (ES) IS CALCULATED USING FORMULA GIVEN BY
        !***  BUCK (1981).  IF Q0 IS ABOVE SATURATION VALUE USING THIS METHOD,
        !***  REDUCE IT TO SATURATION LEVEL.
        
            ES=ALIQ*EXP((BLIQ*T0(K)-CLIQ)/(T0(K)-DLIQ))
            QES(K)=0.622*ES/(P0(K)-ES)
            Q0(K)=AMIN1(QES(K),Q0(K))
        
            IF(Q0(K) <= 0.1E-8)THEN
                Q0(K)=0.1E-8
            ENDIF
        
            RH(K)=Q0(K)/QES(K)
        
        !***  SET HYDROMETEOR CONCENTRATIONS TO ZERO INITIALLY.  ALTHOUGH SOME
        !***  HYDROMETEORS MAY ACTUALLY BE PRESENT, THIS IS DONE TO DISALLOW
        !***  ENTRAINMENT OF THESE INTO CONVECTIVE UPDRAFTS AND DOWNDRAFTS.
        !***  NECESSARY BECAUSE CONVECTIVE SCHEME OPERATES OVER MULTIPLE TIME
        !***  STEPS ASSUMING STEADY-STATE ENVIRONMENTAL CONDITIONS AND THERE
        !***  IS NO WAY OF GUARANTEEING THAT LIQUID WATER WILL CONTINUE TO
        !***  REMAIN AVAILABLE FOR ENTRAINMENT EVEN IF IT IS THERE INITIALLY.
        
            QL0(K)=0.
            QI0(K)=0.
            QR0(K)=0.
            QS0(K)=0.
            DILFRC(K)=1.
        
        !***  CALCULATE WIND AT H-POINT AS AVERAGE OF 4 SURROUNDING V-POINT VALUES
        
            SUMV=VTM(I+IHE(J),J,NK)+VTM(I+IHW(J),J,NK)+VTM(I,J+1,NK)+ &
            VTM(I,J-1,NK)

        ! p - correct for possible HTM/VTM discrepancies
            IF (SUMV > 0) THEN

                U0(K)=(U(I+IHE(J),J,NK)+U(I+IHW(J),J,NK)+U(I,J+1,NK)+ &
                U(I,J-1,NK))/SUMV
                V0(K)=(V(I+IHE(J),J,NK)+V(I+IHW(J),J,NK)+V(I,J+1,NK)+ &
                V(I,J-1,NK))/SUMV

            ELSE

                U0(K)=0.
                V0(K)=0.

            ENDIF
        
            TV0(K)=T0(K)*(1.+0.608*Q0(K))
            RHOE(K)=P0(K)/(R*TV0(K))
        
            DZQ(K)=ROVG*TV0(K)*ALOG((PD(I,J)*RES(I,J)*ETA(NK+1)+PT)/ &
            (PD(I,J)*RES(I,J)*ETA(NK)+PT))
            DP(K)=(ETA(NK+1)-ETA(NK))*PD(I,J)*RES(I,J)
            W0(K)=W0AVG(I,J,NK)
        
        !***  DZQ IS DZ BETWEEN ETA SURFACES. DZA IS DZ BETWEEN MODEL HALF LEVEL.
        !***  DP IS THE PRESSURE INTERVAL BETWEEN FULL ETA LEVELS.
        
            IF(P0(K) >= 500E2)L5=K
            IF(P0(K) >= 400E2)L4=K
            IF(P0(K) >= P300)LLFC=K
            IF(T0(K) > T00)ML=K
            CLDHGT(K)=0.
        
        15 END DO
    !----------------------------------------------------------------------
    
    !***  FILL REMAINING LAYERS WITH ZEROES
    
        DO K=KL+1,KX
            P0(K)=0.
            T0(K)=0.
            Q0(K)=0.
            QES(K)=0.
            U0(K)=0.
            V0(K)=0.
            QL0(K)=0.
            QI0(K)=0.
            QR0(K)=0.
            QS0(K)=0.
            TV0(K)=0.
            RHOE(K)=0.
            DZQ(K)=0.
            DP(K)=0.
            Z00(K)=0.
            W0(K)=0.
            DZA(K)=0.
            DILFRC(K)=0.
        ENDDO
    
        Z00(1)=.5*DZQ(1)
    
        DO K=2,KL
            Z00(K)=Z00(K-1)+0.5*(DZQ(K)+DZQ(K-1))
            DZA(K-1)=Z00(K)-Z00(K-1)
        ENDDO
    
        DZA(KL)=0.
        NLOOP=0.
    
        KMIX=LSB(I)
    !----------------------------------------------------------------------
        25 LOW=KMIX
        NLOOP=NLOOP+1
    
        IF(NLOOP > 50)THEN
            PRINT*,'I, J, LOW, ISHALL, NCHM =',I,J,LOW,ISHALL,NCHM
            IF(NLOOP > 100)THEN
                GO TO 600
                STOP 'NLOOP'
            ENDIF
        ENDIF
    
    !***  IF PARCEL ORIGINATES FROM 300 MB ABOVE SURFACE OR HIGHER THEN SCHEME
    !***  IS NOT USED - CHECK NEXT GRID POINT
    
    !***  VARIABLES FOR SHALLOW CONVECTION
    
        IF(LOW > LLFC)THEN
            IF(ISHALL == 1)THEN
                CHMAX=0.
                NCHM=0
            
                DO NK=1,LLFC
                    IF(CLDHGT(NK) > CHMAX)THEN
                        NCHM=NK
                        CHMAX=CLDHGT(NK)
                    ENDIF
                ENDDO
            
                KMIX=NCHM
                GO TO 25
            ENDIF
            GO TO 900
        ENDIF
    !----------------------------------------------------------------------
    !***
        LC=LOW
        IF(ISHALL == 1 .AND. LC == NCHM)THEN
            FBFRC=1.
        
        !***  COMMENT OUT STATEMENT BELOW SO THAT SHALLOW CLOUD DRAWS
        !***  FROM SAME LAYER AS DEEP CLOUD TO PREVENT SHALLOW CLOUD
        !***  FROM BECOMING TOO DEEP!
        
        !       DPMIN=2.5E3
        ENDIF
    !----------------------------------------------------------------------
    
    !***  ASSUME THAT IN ORDER TO SUPPORT A DEEP UPDRAFT YOU NEED A LAYER OF
    !***  UNSTABLE AIR 50 TO 100 MB DEEP.  TO APPROXIMATE THIS, ISOLATE A
    !***  GROUP OF ADJACENT INDIVIDUAL MODEL LAYERS, WITH THE BASE AT LEVEL
    !***  LC, SUCH THAT THE COMBINED DEPTH OF THESE LAYERS IS AT LEAST 60 MB.
    
    !***  INTRODUCE A MOISTURE PERTURBATION IN UPDRAFT SOURCE LAYERS
    !***  IF RH > 75%.  NOTE THAT THIS APPROACH SHOULD PERHAPS BE
    !***  MODIFIED IF THE GRID-SCALE CONDENSATION THRESHOLD IS CHANGED.
    
        NLAYRS=0
        DPTHMX=0.
    
        DO NK=LC,KX
            DPTHMX=DPTHMX+DP(NK)
            NLAYRS=NLAYRS+1
            QEF=0.
            QUER(NK)=Q0(NK)*(1.+QEF)
            IF(DPTHMX > DPMIN)GO TO 50
        ENDDO
    
        GO TO 900
        50 KPBL=LC+NLAYRS-1
    
    !***  DETERMINE WHAT LEVEL TO START WITH FOR THE NEXT
    !***  MIXTURE IN CASE THE CURRENT MIXTURE, WITH BASE AT
    !***  LEVEL LC, IS NOT BUOYANT.
    !***  INSTEAD OF CHECKING MIXTURES USING EVERY SINGLE LAYER,
    !***  MOVE UP IN INCREMENTS OF AT LEAST 20 MB.
    
        PM15=P0(LC)-15.E2
    
        DO NK=LC+1,KL
            IF(P0(NK) < PM15)THEN
                KMIX=NK
                GO TO 75
            ENDIF
        ENDDO
    
        GO TO 900
        75 CONTINUE
    !----------------------------------------------------------------------
    !***
    !***  FOR COMPUTATIONAL SIMPLICITY WITHOUT MUCH LOSS IN ACCURACY,
    !***  MIX TEMPERATURE INSTEAD OF THETA FOR EVALUATING CONVECTIVE
    !***  INITIATION (TRIGGERING) POTENTIAL.
    
        TMIX=0.
        QMIX=0.
        ZMIX=0.
        PMIX=0.
    
    !***  FIND THE THERMODYNAMIC CHARACTERISTICS OF THE LAYER BY
    !***  MASS-WEIGHTING THE CHARACTERISTICS OF THE INDIVIDUAL MODEL
    !***  LAYERS.
    
        DO NK=LC,KPBL
            TMIX=TMIX+DP(NK)*T0(NK)
            QMIX=QMIX+DP(NK)*QUER(NK)
            ZMIX=ZMIX+DP(NK)*Z00(NK)
            PMIX=PMIX+DP(NK)*P0(NK)
        ENDDO
    
        TMIX=TMIX/DPTHMX
        QMIX=QMIX/DPTHMX
        ZMIX=ZMIX/DPTHMX
        PMIX=PMIX/DPTHMX
        EMIX=QMIX*PMIX/(0.622+QMIX)
    
    !***  FIND THE TEMPERATURE OF THE MIXTURE AT ITS LCL
    
        ASTRT=1.E-3
        AINC=0.075
        A1=EMIX/ALIQ
        TP=(A1-ASTRT)/AINC
        INDLU=INT(TP)+1
        VALUE=(INDLU-1)*AINC+ASTRT
        AINTRP=(A1-VALUE)/AINC
        TLOG=AINTRP*ALU(INDLU+1)+(1-AINTRP)*ALU(INDLU)
        TDPT=(CLIQ-DLIQ*TLOG)/(BLIQ-TLOG)
        TLCL=TDPT-(0.212+1.571E-3*(TDPT-T00)-4.36E-4*(TMIX-T00)) &
        *(TMIX-TDPT)
        TLCL=AMIN1(TLCL,TMIX)
        TVLCL=TLCL*(1.+0.608*QMIX)
    ! lim      TVQU(K)=TVLCL
        ZLCL=ZMIX+(TLCL-TMIX)/GDRY
    
        DO NK=LC,KL
            KLCL=NK
            IF(ZLCL <= Z00(NK))GO TO 100
        ENDDO
    
        GO TO 900
        100 K=KLCL-1
    !----------------------------------------------------------------------
    
    !***  CALCULATE DLP USING Z INSTEAD OF LOG(P)
    
        DLP=(ZLCL-Z00(K))/(Z00(KLCL)-Z00(K))
    
    !***  ESTIMATE ENVIRONMENTAL TEMPERATURE AND MIXING RATIO AT THE LCL.
    
        TENV=T0(K)+(T0(KLCL)-T0(K))*DLP
        QENV=Q0(K)+(Q0(KLCL)-Q0(K))*DLP
        TVEN=TENV*(1.+0.608*QENV)
    
    !***  CHECK TO SEE IF CLOUD IS BUOYANT USING FRITSCH-CHAPPELL TRIGGER
    !***  FUNCTION DESCRIBED IN KAIN AND FRITSCH (1992).  W0 IS AN
    !***  APROXIMATE VALUE FOR THE RUNNING-MEAN GRID-SCALE VERTICAL
    !***  VELOCITY, WHICH GIVES SMOOTHER FIELDS OF CONVECTIVE INITIATION
    !***  THAN THE INSTANTANEOUS VALUE.  FORMULA RELATING TEMPERATURE
    !***  PERTURBATION TO VERTICAL VELOCITY HAS BEEN USED WITH THE MOST
    !***  SUCCESS AT GRID LENGTHS NEAR 25 KM.  FOR DIFFERENT GRID-LENGTHS,
    !***  ADJUST VERTICAL VELOCITY TO EQUIVALENT VALUE FOR 25 KM GRID
    !***  LENGTH, ASSUMING LINEAR DEPENDENCE OF W ON GRID LENGTH.
    
        IF(ZLCL < 2.E3)THEN
            WKLCL=0.02*ZLCL/2.E3
        ELSE
            WKLCL=0.02
        ENDIF
    
        WKL=(W0(K)+(W0(KLCL)-W0(K))*DLP)*DX(I,J) &
        /25.E3-WKLCL
        IF(WKL < 0.0001)THEN
            DTLCL=0.
        ELSE
            DTLCL=4.64*WKL**0.33
        ENDIF
    !----------------------------------------------------------------------
    
    !***  GIVE PARCEL AN EXTRA TEMPERATURE PERTURBATION BASED
    !***  THE THRESHOLD RH FOR CONDENSATION.
    !***  FOR NOW, JUST ASSUME U00=0.75.
    
        U00=0.75
    
        IF(U00 < 1.)THEN
            QSLCL=QES(K)+(QES(KLCL)-QES(K))*DLP
            RHLCL=QENV/QSLCL
            DQSDT=QMIX*(CLIQ-BLIQ*DLIQ)/((TLCL-DLIQ)*(TLCL-DLIQ))
            IF(RHLCL >= 0.75 .AND. RHLCL <= 0.95)THEN
                DTRH=0.25*(RHLCL-0.75)*QMIX/DQSDT
            ELSEIF(RHLCL > 0.95)THEN
                DTRH=(1./RHLCL-1.)*QMIX/DQSDT
            ELSE
                DTRH=0.
            ENDIF
        ENDIF
    
        IF(TLCL+DTLCL+DTRH > TENV)GO TO 150
        IF(ISHALL == 1 .AND. LC == NCHM)GO TO 900
        IF(KMIX <= LLFC)GO TO 25
    
    !***  SHALLOW CONVECTION CHANGES
    
        IF(ISHALL == 1)THEN
            CHMAX=0.
            NCHM=0
        
            DO NK=1,LLFC
                IF(CLDHGT(NK) > CHMAX)THEN
                    NCHM=NK
                    CHMAX=CLDHGT(NK)
                ENDIF
            ENDDO
        
            KMIX=NCHM
            GO TO 25
        ENDIF
    !***
        GO TO 900
        150 CONTINUE
    !----------------------------------------------------------------------
    
    !***  CONVECTIVE TRIGGERING CRITERIA HAS BEEN SATISFIED.
    !***  COMPUTE EQUIVALENT POTENTIAL TEMPERATURE.
    !***  (THETEU) AND VERTICAL VELOCITY OF THE RISING PARCEL AT THE LCL.
    
        THMIX=TMIX*(1.E5/PMIX)**(0.2854*(1.-0.28*QMIX))
        THETEU(K)=THMIX* &
        EXP((3374.6525/TLCL-2.5403)*QMIX*(1.+0.81*QMIX))
        ES=ALIQ*EXP((TENV*BLIQ-CLIQ)/(TENV-DLIQ))
        TVAVG=0.5*(TV0(KLCL)+TENV*(1.+0.608*QENV))
    
    !***  MODIFY CALCULATION OF INITIAL PARCEL VERTICAL VELOCITY
    
        GDT=2.*G*(DTLCL+DTRH)*500./TVEN
        WLCL=1.+0.5*SQRT(ABS(GDT)+1.E-10)
        WLCL=AMIN1(WLCL,3.)
        PLCL=P0(K)+(P0(KLCL)-P0(K))*DLP
        WTW=WLCL*WLCL
        IF(WLCL < 0.)GO TO 25
        TVLCL=TLCL*(1.+0.608*QMIX)
        RHOLCL=PLCL/(R*TVLCL)
        PLCL0=PLCL
    
        LCL=KLCL
        LET=LCL
    
        IF(WKL < 0.)THEN
            RAD=1000.
        ELSEIF(WKL > 0.1)THEN
            RAD=2000.
        ELSE
            RAD=1000.+1.E4*WKL
        ENDIF
    
    !*******************************************************************
    !                                                                  *
    !                 COMPUTE UPDRAFT PROPERTIES                       *
    !                                                                  *
    !*******************************************************************
    
    !***
    !***  ESTIMATE INITIAL UPDRAFT MASS FLUX (UMF(K))
    !***
        WU(K)=WLCL
        AU0=PIE*RAD*RAD
        UMF(K)=RHOLCL*AU0
        VMFLCL=UMF(K)
        UPOLD=VMFLCL
        UPNEW=UPOLD
    
    !***  RATIO2 IS THE DEGREE OF GLACIATION IN THE CLOUD (0 TO 1),
    !***  UER IS THE ENVIR ENTRAINMENT RATE, ABE IS AVAILABLE
    !***  BUOYANT ENERGY, TRPPT IS THE TOTAL RATE OF PRECIPITATION
    !***  PRODUCTION.
    
        RATIO2(K)=0.
        UER(K)=0.
        ABE=0.
        TRPPT=0.
        TU(K)=TLCL
        TVU(K)=TVLCL
        QU(K)=QMIX
        EQFRC(K)=1.
        QLIQ(K)=0.
        QICE(K)=0.
        QLQOUT(K)=0.
        QICOUT(K)=0.
        DETLQ(K)=0.
        DETIC(K)=0.
        PPTLIQ(K)=0.
        PPTICE(K)=0.
        IFLAG=0
        KFRZ=LC
    
    !***  THE AMOUNT OF CONV AVAIL POT ENERGY (CAPE) IS CALCULATED WITH
    !***  RESPECT TO UNDILUTE PARCEL ASCENT; EQ POT TEMP OF UNDILUTE
    !***  PARCEL IS THTUDL, UNDILUTE TEMPERATURE IS GIVEN BY TUDL.
    
        THTUDL=THETEU(K)
        TUDL=TLCL
    
    !***  TTEMP IS USED DURING CALCULATION OF THE LINEAR GLACIATION
    !***  PROCESS; IT IS INITIALLY SET TO THE TEMPERATURE AT WHICH
    !***  FREEZING IS SPECIFIED TO BEGIN.  WITHIN THE GLACIATION
    !***  INTERVAL, IT IS SET EQUAL TO THE UPDRAFT TEMP AT THE
    !***  PREVIOUS MODEL LEVEL.
    
        TTEMP=TTFRZ
    
    !***  ENTER THE LOOP FOR UPDRAFT CALCULATIONS.  CALCULATE UPDRAFT TEMP,
    !***  MIXING RATIO, VERTICAL MASS FLUX, LATERAL DETRAINMENT OF MASS AND
    !***  MOISTURE, PRECIPITATION RATES AT EACH MODEL LEVEL.
    
    
    !***  DIAGNOSTIC STUFF
    
        PLCL0=PLCL
        LFC=0
        PMIX0=PMIX
        REI=0.
        DILBE=0.
        UDLBE=0.
    
    !----------------------------------------------------------------------
    !----------------------------------------------------------------------
        DO 250 NK=K,KLM
            NK1=NK+1
            RATIO2(NK1)=RATIO2(NK)
            FRC1=0.
            TU(NK1)=T0(NK1)
            THETEU(NK1)=THETEU(NK)
            QU(NK1)=QU(NK)
            QLIQ(NK1)=QLIQ(NK)
            QICE(NK1)=QICE(NK)
        
            CALL TPMIX2(P0(NK1),THETEU(NK1),TU(NK1),QU(NK1),QLIQ(NK1), &
            QICE(NK1),QNEWLQ,QNEWIC,RATIO2(NK1), &
            XLV1,XLV0)
        !----------------------------------------------------------------------
        
        !***  CHECK TO SEE IF UPDRAFT TEMP IS ABOVE THE TEMPERATURE AT WHICH
        !***  GLACIATION IS ASSUMED TO INITIATE; IF IT IS, CALCULATE THE
        !***  FRACTION OF REMAINING LIQUID WATER TO FREEZE.  FRZ IS THE
        !***  TEMP AT WHICH FREEZING BEGINS, TBFRZ THE TEMP BELOW WHICH ALL
        !***  LIQUID WATER IS FROZEN AT EACH LEVEL.
        
            IF(TU(NK1) <= TTFRZ)THEN
            
                IF(TU(NK1) > TBFRZ)THEN
                    IF(TTEMP > TTFRZ)TTEMP=TTFRZ
                    FRC1=(TTEMP-TU(NK1))/(TTEMP-TBFRZ)
                ELSE
                    FRC1=1.
                    IFLAG=1
                ENDIF
            
                TTEMP=TU(NK1)
            
            !***  DETERMINE THE EFFECTS OF LIQUID WATER FREEZING WHEN TEMPERATURE
            !***  IS BELOW TTFRZ
            
                QFRZ=(QLIQ(NK1)+QNEWLQ)*FRC1
                QNEWIC=QNEWIC+QNEWLQ*FRC1
                QNEWLQ=QNEWLQ-QNEWLQ*FRC1
                QICE(NK1)=QICE(NK1)+QLIQ(NK1)*FRC1
                QLIQ(NK1)=QLIQ(NK1)-QLIQ(NK1)*FRC1
                CALL DTFRZNEW(TU(NK1),P0(NK1),THETEU(NK1),QU(NK1),QFRZ, &
                QICE(NK1),ALIQ,BLIQ,CLIQ,DLIQ)
            ENDIF
        !----------------------------------------------------------------------
        
            TVU(NK1)=TU(NK1)*(1.+0.608*QU(NK1))
        
        !***  CALCULATE UPDRAFT VERTICAL VELOCITY AND PRECIPITATION FALLOUT...
        
            IF(NK == K)THEN
                BE=(TVLCL+TVU(NK1))/(TVEN+TV0(NK1))-1.
                BOTERM=2.*(Z00(NK1)-ZLCL)*G*BE/1.5
                DZZ=Z00(NK1)-ZLCL
            ELSE
                BE=(TVU(NK)+TVU(NK1))/(TV0(NK)+TV0(NK1))-1.
                BOTERM=2.*DZA(NK)*G*BE/1.5
                DZZ=DZA(NK)
            ENDIF
        
            ENTERM=2.*REI*WTW/UPOLD
        
        !***  DIAGNOSTICS
        
            IF(TVU(NK1) > TV0(NK1))THEN
                IF(TVU(NK) < TV0(NK) .OR. NK1 == KLCL)LFC=NK1
            ENDIF
        
            WSQ=WTW
        
        !----------------------------------------------------------------------
            CALL CONDLOAD(QLIQ(NK1),QICE(NK1),WTW,DZZ,BOTERM,ENTERM, &
            RATE,QNEWLQ,QNEWIC,QLQOUT(NK1),QICOUT(NK1))
        !----------------------------------------------------------------------
        
            WABS=SQRT(ABS(WTW))
            WU(NK1)=WTW/WABS
        
        !***  IF VERT VELOCITY IS LESS THAN ZERO, EXIT THE UPDRAFT LOOP AND,
        !***  IF CLOUD IS TALL ENOUGH, FINALIZE UPDRAFT CALCULATIONS.
        
            IF(WU(NK1) < 0.)GO TO 275
        
        !***  CALL SUBROUTINE TO CALCULATE ENVIRONMENTAL EQUIVALENT POTENTIAL
        !***  TEMP...WITHIN GLACIATION INTERVAL, THETAE MUST BE CALCULATED
        !***  WITH RESPECT TO THE SAME DEGREE OF GLACIATION FOR ALL ENTRAINING
        !***  AIR.
        
        !***  FOR LOOKUP TABLE VERSION, CALCULATE THETAE WITH RESPECT TO
        !***  LIQUID WATER AT ALL LEVELS.
        
            IF(NK1 <= KPBL)THEN
                CALL ENVIRTHT(P0(NK1),T0(NK1),QUER(NK1),THETEE(NK1),0., &
                RL,ALIQ,BLIQ,CLIQ,DLIQ,AICE,BICE,CICE,DICE)
            ELSE
                CALL ENVIRTHT(P0(NK1),T0(NK1),Q0(NK1),THETEE(NK1),0., &
                RL,ALIQ,BLIQ,CLIQ,DLIQ,AICE,BICE,CICE,DICE)
            ENDIF
        
        !***  REI IS THE RATE OF ENVIRONMENTAL INFLOW
        
            REI=VMFLCL*DP(NK1)*0.03/RAD
            TVQU(NK1)=TU(NK1)*(1.+0.608*QU(NK1)-QLIQ(NK1)-QICE(NK1))
        
            IF(NK == K)THEN
                DILBE=((TVLCL+TVQU(NK1))/(TVEN+TV0(NK1))-1.)*DZZ
            ELSE
                DILBE=((TVQU(NK)+TVQU(NK1))/(TV0(NK)+TV0(NK1))-1.)*DZZ
            ENDIF
        
            IF(DILBE > 0.)ABE=ABE+DILBE*G
        !----------------------------------------------------------------------
        
        !***  IF CLOUD PARCELS ARE VIRTUALLY COLDER THAN THE ENVIRONMENT,
        !***  NO ENTRAINMENT IS ALLOWED AT THIS LEVEL.
        
            IF(TVQU(NK1) <= TV0(NK1))THEN
            
            !***  USE A MINIMUM ENTRAINMENT RATE (UER) OF 0.5*REI
            
                UER(NK1)=0.5*REI
                UDR(NK1)=1.5*REI
                EE2=0.5
                UD2=1.
                EQFRC(NK1)=0.
                GO TO 200
            ENDIF
        
            LET=NK1
            TTMP=TVQU(NK1)
        
        !***  DETERMINE THE CRITICAL MIXED FRACTION OF UPDRAFT
        !***  AND ENVIRONMENTAL AIR
        
            F1=0.95
            F2=1.-F1
            THTTMP=F1*THETEE(NK1)+F2*THETEU(NK1)
            QTMP=F1*Q0(NK1)+F2*QU(NK1)
            TMPLIQ=F2*QLIQ(NK1)
            TMPICE=F2*QICE(NK1)

            CALL TPMIX2(P0(NK1),THTTMP,TTMP,QTMP,TMPLIQ,TMPICE, &
            QNEWLQ,QNEWIC,RATIO2(NK1), &
            XLV1,XLV0)
        
            TU95=TTMP*(1.+0.608*QTMP-TMPLIQ-TMPICE)
        
            IF(TU95 > TV0(NK1))THEN
                EE2=1.
                UD2=0.
                EQFRC(NK1)=1.
                GO TO 175
            ENDIF
        
            F1=0.10
            F2=1.-F1
            THTTMP=F1*THETEE(NK1)+F2*THETEU(NK1)
            QTMP=F1*Q0(NK1)+F2*QU(NK1)
            TMPLIQ=F2*QLIQ(NK1)
            TMPICE=F2*QICE(NK1)
        
            CALL TPMIX2(P0(NK1),THTTMP,TTMP,QTMP,TMPLIQ,TMPICE, &
            QNEWLQ,QNEWIC,RATIO2(NK1), &
            XLV1,XLV0)
        
            TU10=TTMP*(1.+0.608*QTMP-TMPLIQ-TMPICE)
        
            IF(TU10 == TVQU(NK1))THEN
                EE2=1.
                UD2=0.
                EQFRC(NK1)=1.0
                GO TO 175
            ENDIF
        
            EQFRC(NK1)=(TV0(NK1)-TVQU(NK1))*F1/(TU10-TVQU(NK1))
            EQFRC(NK1)=AMAX1(0.0,EQFRC(NK1))
            EQFRC(NK1)=AMIN1(1.0,EQFRC(NK1))
        
            IF(EQFRC(NK1) == 1)THEN
                EE2=1.
                UD2=0.
                GO TO 175
            ELSEIF(EQFRC(NK1) == 0.)THEN
                EE2=0.
                UD2=1.
                GO TO 175
            ELSE
            
            !***  SUBROUTINE PROF5 INTEGRATES OVER THE GAUSSIAN DIST TO DETERMINE THE
            !***  FRACTIONAL ENTRAINMENT AND DETRAINMENT RATES
            
                CALL PROF5(EQFRC(NK1),EE2,UD2)
            ENDIF
        
            175 CONTINUE
        !----------------------------------------------------------------------
        
            IF(NK == K)THEN
                EE1=1.
                UD1=0.
            ENDIF
        
        !***  NET ENTRAINMENT AND DETRAINMENT RATES ARE GIVEN BY THE AVERAGE
        !***  FRACTIONAL VALUES IN THE LAYER
        
        !***  USE A MINIMUM ENTRAINMENT RATE (UER) OF 0.5*REI
        
            EE2=AMAX1(EE2,0.5)
            UD2=1.5*UD2
            UER(NK1)=0.5*REI*(EE1+EE2)
            UDR(NK1)=0.5*REI*(UD1+UD2)
        !----------------------------------------------------------------------
        
        !***  IF THE CALCULATED UPDRAFT DETRAINMENT RATE IS GREATER THAN THE TOTAL
        !***  UPDRAFT MASS FLUX, ALL CLOUD MASS DETRAINS, EXIT UPDRAFT CALCULATIONS.
        
            200 IF(UMF(NK)-UDR(NK1) < 10.)THEN
            
            !***  IF THE CALCULATED DETRAINED MASS FLUX IS GREATER THAN THE TOTAL
            !***  UPWARD MASS FLUX, IMPOSE TOTAL DETRAINMENT OF UPDRAFT MASS AT
            !***  THE PREVIOUS MODEL LVL
            
                IF(DILBE > 0.)ABE=ABE-DILBE*G
                LET=NK
                GO TO 275
            ENDIF
        
            EE1=EE2
            UD1=UD2
            UPOLD=UMF(NK)-UDR(NK1)
            UPNEW=UPOLD+UER(NK1)
            UMF(NK1)=UPNEW
            DILFRC(NK1)=UPNEW/UPOLD
        
        !***  DETLQ AND DETIC ARE THE RATES OF DETRAINMENT OF LIQUID AND
        !***  ICE IN THE DETRAINING UPDRAFT MASS
        
            DETLQ(NK1)=QLIQ(NK1)*UDR(NK1)
            DETIC(NK1)=QICE(NK1)*UDR(NK1)
            QDT(NK1)=QU(NK1)
        
            IF(NK1 <= KPBL)THEN
                QU(NK1)=(UPOLD*QU(NK1)+UER(NK1)*QUER(NK1))/UPNEW
            ELSE
                QU(NK1)=(UPOLD*QU(NK1)+UER(NK1)*Q0(NK1))/UPNEW
            ENDIF
        
            THETEU(NK1)=(THETEU(NK1)*UPOLD+THETEE(NK1)*UER(NK1))/UPNEW
            QLIQ(NK1)=QLIQ(NK1)*UPOLD/UPNEW
            QICE(NK1)=QICE(NK1)*UPOLD/UPNEW
        
        !***  KFRZ IS THE HIGHEST MODEL LEVEL AT WHICH LIQUID CONDENSATE
        !***  IS GENERATED...PPTLIQ IS THE RATE OF GENERATION (FALLOUT) OF
        !***  LIQUID PRECIP AT A GIVEN MODEL LVL, PPTICE THE SAME FOR ICE,
        !***  TRPPT IS THE TOTAL RATE OF PRODUCTION OF PRECIP UP TO THE
        !***  CURRENT MODEL LEVEL...
        
            IF(ABS(RATIO2(NK1)-1.) > 1.E-6)KFRZ=NK1
        
        !***  REVERSE THE MOD THAT ALLOWS FEEDBACK OF RAIN/SNOW THAT ORIGINATES
        !***  IN DETRAINED AIR
        
            PPTLIQ(NK1)=QLQOUT(NK1)*UMF(NK)
            PPTICE(NK1)=QICOUT(NK1)*UMF(NK)
        
            TRPPT=TRPPT+PPTLIQ(NK1)+PPTICE(NK1)
            IF(NK1 <= KPBL)UER(NK1)=UER(NK1)+VMFLCL*DP(NK1)/DPTHMX
        250 END DO
    !----------------------------------------------------------------------
    
    !***  CHECK CLOUD DEPTH.  IF CLOUD IS TALL ENOUGH, ESTIMATE THE
    !***  EQUILIBRIUM TEMPERATURE LEVEL (LET) AND ADJUST MASS FLUX PROFILE
    !***  AT CLOUD TOP SO THAT MASS FLUX DECREASES TO ZERO AS A LINEAR
    !***  FUNCTION OF PRESSURE BETWEEN THE LET AND CLOUD TOP.
    
    !***  LTOP IS THE MODEL LEVEL JUST BELOW THE LEVEL AT WHICH VERTICAL
    !***  VELOCITY FIRST BECOMES NEGATIVE
    
        275 LTOP=NK
        CLDHGT(LC)=Z00(LTOP)-ZLCL
    
    !***  IF CLOUD TOP HEIGHT IS LESS THAN THE SPECIFIED MINIMUM FOR DEEP
    !***  CONVECTION, SAVE VALUE TO CONSIDER THIS LEVEL AS SOURCE FOR
    !***  SHALLOW CONVECTION, GO BACK UP TO CHECK NEXT LEVEL.
    
    !***  TRY SPECIFYING MINIMUM CLOUD DEPTH AS A FUNCTION OF TLCL
    
        IF(TLCL > 293.)THEN
            CHMIN=4.E3
        ELSEIF(TLCL <= 293. .AND. TLCL >= 273)THEN
            CHMIN=2.E3 + 100.*(TLCL-273.)
        ELSEIF(TLCL < 273.)THEN
            CHMIN=2.E3
        ENDIF
    
        KSTART=MAX0(KPBL,KLCL)
    
    !----------------------------------------------------------------------
    !***  DO NOT ALLOW ANY CLOUD FROM THIS LAYER IF:
    
    !   1.) IF THERE IS NO CAPE, OR
    !   2.) CLOUD TOP IS AT MODEL LEVEL JUST ABOVE LCL, OR
    !   3.) CLOUD TOP IS WITHIN UPDRAFT SOURCE LAYER, OR
    !   4.) CLOUD-TOP DETRAINMENT LAYER BEGINS WITHIN
    !       UPDRAFT SOURCE LAYER
    
        IF(LTOP <= KLCL .OR. &
        LTOP <= KPBL .OR. LET+1 <= KPBL)THEN
            CLDHGT(LC)=0.
        
        !***  IF THIS IS SELECTED SHALLOW SOURCE AND STILL DOES NOT MAKE A
        !***  SIGNIFICANT CLOUD, GO ON TO NEXT GRID POINT.
        
            IF(LC == NCHM)GO TO 900
        
        !***  IF ALL LAYERS IN THE SPECIFIED PORTION OF LOWER ATMOSPHERE
        !***  HAVE BEEN CHECKED, THEN...
        
            IF(KMIX > LLFC)THEN
            
            !***  IF NO POSSIBLE SHALLOW CLOUD LAYERS WERE FOUND, GO TO NEXT
            !***  GRID POINT
            
                IF(ISHALL == 0)THEN
                    GO TO 900
                ELSE
                
                !***  IF SOME POTENTIAL SHALLOW CLOUD SOURCE LAYERS WERE FOUND,
                !***  FIND THE ONE THAT GIVES THE TALLEST CLOUD.
                
                    GO TO 305
                ENDIF
            ELSE
            
            !***  IF THERE ARE MORE LAYERS TO CHECK, RESET CLOUD CHARACTERISTICS,
            !***  GO TO NEXT LEVEL.
                GO TO 310
            ENDIF
        
        !***  IF THIS LAYER HAS BEEN SELECTED AS A SHALLOW CONVECTIVE SOURCE,
        !***  ALLOW SHALLOW CONVECTION.  (EVEN IF THE SHALLOW-CLOUD PARAMETERS
        !***  ALLOW CLDHGT TO EXCEED CHMIN)
        
        ELSEIF(LC == NCHM)THEN
            GO TO 315
        
        !***  IF CLOUD DEPTH IS GREATER THAN MINIMUM DEPTH CRITERION, AND
        !***  THIS LAYER HAS NOT BEEN MARKED AS A SHALLOW CONVECTIVE SOURCE LAYER,
        !***  ALLOW DEEP CONVECTION.
        
        ELSEIF(CLDHGT(LC) > CHMIN .AND. ABE > 1)THEN
            ISHALL=0
            GO TO 315
        ENDIF
    
    !***  AT THIS POINT, WE HAVE A PARCEL THAT IS ABLE TO MAINTAIN UPWARD
    !***  MOMENTUM FOR AT LEAST A SHORT DISTANCE, BUT THE CLOUD FROM THIS
    !***  SOURCE LAYER (LC) IS NOT DEEP ENOUGH FOR "DEEP" (PRECIPITATING)
    !***  CONVECTION.  SET ISHALL=1 TO SAVE LC AS A POSSIBLE FOR SOURCE
    !***  LAYER FOR SHALLOW CONVECTION.
    
    !----------------------------------------------------------------------
    !----------------------------------------------------------------------
    
    !***  TO DISALLOW SHALLOW CONVECTION, COMMENT OUT NEXT LINE !!!!!!!!
    
        ISHALL=1
    !----------------------------------------------------------------------
    !----------------------------------------------------------------------
        IF(KMIX > LLFC)THEN
            GO TO 305
        ELSE
            GO TO 310
        ENDIF
    
        305 CONTINUE
    
        IF(ISHALL == 0)THEN
            GO TO 900
        ELSE
            CHMAX=0.
            NCHM=0
            DO NK=1,LLFC
                IF(CLDHGT(NK) > CHMAX)THEN
                    NCHM=NK
                    CHMAX=CLDHGT(NK)
                ENDIF
            ENDDO
            KMIX=NCHM
        ENDIF
    
        310 CONTINUE
    
        DO NK=K,LTOP
            UMF(NK)=0.
            UDR(NK)=0.
            UER(NK)=0.
            DETLQ(NK)=0.
            DETIC(NK)=0.
            PPTLIQ(NK)=0.
            PPTICE(NK)=0.
        ENDDO
    
        GO TO 25
    
        315 CONTINUE
        IF(ISHALL == 1)THEN
            KSTART=MAX0(KPBL,KLCL)
            LET=KSTART
        ENDIF
    !----------------------------------------------------------------------
    
    !***  IF THE LET AND LTOP ARE THE SAME, DETRAIN ALL OF THE UPDRAFT MASS
    !***  FLUX AT THIS LEVEL.
    
        IF(LET == LTOP)THEN
            UDR(LTOP)=UMF(LTOP)+UDR(LTOP)-UER(LTOP)
            DETLQ(LTOP)=QLIQ(LTOP)*UDR(LTOP)*UPNEW/UPOLD
            DETIC(LTOP)=QICE(LTOP)*UDR(LTOP)*UPNEW/UPOLD
        
        !***  REVERSE THE MOD THAT ALLOWS FEEDBACK OF RAIN/SNOW THAT ORIGINATES
        !***  IN DETRAINED AIR.
        
            UER(LTOP)=0.
            UMF(LTOP)=0.
            GO TO 350
        ENDIF
    
    !***  BEGIN TOTAL DETRAINMENT AT THE LEVEL ABOVE THE LET
    
        DPTT=0.
    
        DO NJ=LET+1,LTOP
            DPTT=DPTT+DP(NJ)
        ENDDO
    
        DUMFDP=UMF(LET)/DPTT
    !----------------------------------------------------------------------
    
    !***  ADJUST MASS FLUX PROFILES, DETRAINMENT RATES, AND PRECIPITATION FALL
    !***  RATES TO REFLECT THE LINEAR DECREASE IN MASS FLX BETWEEN THE LET AND
    
        DO 325 NK=LET+1,LTOP
        
        !***  ENTRAINMENT IS ALLOWED AT EVERY LEVEL EXCEPT FOR LTOP, SO DISALLOW
        !***  ENTRAINMENT AT LTOP AND ADJUST ENTRAINMENT RATES BETWEEN LET AND LTOP
        !***  SO THE THE DILUTION FACTOR DUE TO ENTRAINMENT IS NOT CHANGED BUT
        !***  THE ACTUAL ENTRAINMENT RATE WILL CHANGE DUE DUE FORCED TOTAL
        !***  DETRAINMENT IN THIS LAYER.
        
            IF(NK == LTOP)THEN
                UDR(NK)=UMF(NK-1)
                UER(NK)=0.
                DETLQ(NK)=UDR(NK)*QLIQ(NK)*DILFRC(NK)
                DETIC(NK)=UDR(NK)*QICE(NK)*DILFRC(NK)
            ELSE
                UMF(NK)=UMF(NK-1)-DP(NK)*DUMFDP
                UER(NK)=UMF(NK)*(1.-1./DILFRC(NK))
                UDR(NK)=UMF(NK-1)-UMF(NK)+UER(NK)
                DETLQ(NK)=UDR(NK)*QLIQ(NK)*DILFRC(NK)
                DETIC(NK)=UDR(NK)*QICE(NK)*DILFRC(NK)
            ENDIF
        
        !***  REVERSE THE MOD THAT ALLOWS FEEDBACK OF RAIN/SNOW THAT ORIGINATES
        !***  IN DETRAINED AIR.
        
            IF(NK >= LET+2)THEN
                TRPPT=TRPPT-PPTLIQ(NK)-PPTICE(NK)
                PPTLIQ(NK)=UMF(NK-1)*QLQOUT(NK)
                PPTICE(NK)=UMF(NK-1)*QICOUT(NK)
                TRPPT=TRPPT+PPTLIQ(NK)+PPTICE(NK)
            ENDIF
        
        325 END DO
    
        350 CONTINUE
    !----------------------------------------------------------------------
    
    !***  SEND UPDRAFT CHARACTERISTICS TO OUTPUT FILES
    
    !***  WHEN MOIST PERTURBATION IS ADDED TO UPDRAFT AND KLCL.LE.KPBL,
    !***  RESET THETEE SO THAT MOISTURE PERTURBATION ONLY AFFECTS UPDRAFTS.
    
        IF(KLCL <= KPBL)THEN
            DO NK1=KLCL,KPBl
                CALL ENVIRTHT(P0(NK1),T0(NK1),Q0(NK1),THETEE(NK1),0., &
                RL,ALIQ,BLIQ,CLIQ,DLIQ,AICE,BICE,CICE,DICE)
            ENDDO
        ENDIF
    
        XTIME=NTSD*DT/60.
    
    !***  EXTEND THE UPDRAFT MASS FLUX PROFILE DOWN TO THE SOURCE LAYER FOR
    !***  THE UPDRAFT AIR.  ALSO, DEFINE THETAE FOR LEVELS BELOW THE LCL.
    
        DO 360 NK=1,K
        
            IF(NK >= LC)THEN
                IF(NK == LC)THEN
                    UMF(NK)=VMFLCL*DP(NK)/DPTHMX
                    UER(NK)=VMFLCL*DP(NK)/DPTHMX
                ELSEIF(NK <= KPBL)THEN
                    UER(NK)=VMFLCL*DP(NK)/DPTHMX
                    UMF(NK)=UMF(NK-1)+UER(NK)
                ELSE
                    UMF(NK)=VMFLCL
                    UER(NK)=0.
                ENDIF
                TU(NK)=TMIX+(Z00(NK)-ZMIX)*GDRY
                QU(NK)=QMIX
                WU(NK)=WLCL
            ELSE
                TU(NK)=0.
                QU(NK)=0.
                UMF(NK)=0.
                WU(NK)=0.
                UER(NK)=0.
            ENDIF
        
            UDR(NK)=0.
            QDT(NK)=0.
            QLIQ(NK)=0.
            QICE(NK)=0.
            QLQOUT(NK)=0.
            QICOUT(NK)=0.
            PPTLIQ(NK)=0.
            PPTICE(NK)=0.
            DETLQ(NK)=0.
            DETIC(NK)=0.
            RATIO2(NK)=0.
            EE=Q0(NK)*P0(NK)/(0.622+Q0(NK))
            TLOG=ALOG(EE/ALIQ)
            TDPT=(CLIQ-DLIQ*TLOG)/(BLIQ-TLOG)
            TSAT=TDPT-(.212+1.571E-3*(TDPT-T00)-4.36E-4*(T0(NK)-T00))*( &
            T0(NK)-TDPT)
            THTA=T0(NK)*(1.E5/P0(NK))**(0.2854*(1.-0.28*Q0(NK)))
            THETEE(NK)=THTA* &
            EXP((3374.6525/TSAT-2.5403)*Q0(NK)*(1.+0.81*Q0(NK)))
            THTES(NK)=THTA* &
            EXP((3374.6525/T0(NK)-2.5403)*QES(NK)*(1.+0.81* &
            QES(NK)))
            EQFRC(NK)=1.0
        360 END DO
    !----------------------------------------------------------------------
    
    !***  FIND HEIGHT OF LFC, CALCULATE CIN.
    
        IF(LFC > 0)THEN
            CIN1=0.
            IF(LFC == KLCL)THEN
                IF(TVLCL > TVEN)THEN
                    CIN1=0.
                ELSE
                    Z00LFC=Z00(LFC)
                    IF(Z00LFC /= ZLCL)THEN
                        DTUDZ=(TVU(LFC)-TVLCL)/(Z00LFC-ZLCL)
                        DTEDZ=(TV0(LFC)-TVEN )/(Z00LFC-ZLCL)
                        ZLFC=ZLCL+(TVEN-TVLCL)/(DTUDZ-DTEDZ)
                        TVLFC=TVEN+(ZLFC-ZLCL)*DTEDZ
                    ELSE
                        ZLFC=ZLCL
                        TVLFC=TVEN
                    ENDIF
                    CIN1=G*(ZLFC-ZLCL)*(TVLCL-TVEN)/(TVEN+TVLFC)
                ENDIF
            ELSE
                CIN1=CIN1+ &
                G*(Z00(KLCL)-ZLCL)*((TVU(KLCL)+TVLCL)/ &
                (TV0(KLCL)+TVEN)-1.)
                DO NK=KLCL+1,LFC
                    IF(NK == LFC)THEN
                        DTUDZ=(TVU(LFC)-TVU(LFC-1))/(Z00(LFC)-Z00(LFC-1))
                        DTEDZ=(TV0(LFC)-TV0(LFC-1))/(Z00(LFC)-Z00(LFC-1))
                        ZLFC=Z00(LFC-1)+(TV0(LFC-1)-TV0(LFC-1))/(DTUDZ-DTEDZ)
                        TVEN=TV0(LFC-1)+(ZLFC-Z00(LFC-1))*DTEDZ
                        CIN1=CIN1+G*(ZLFC-Z00(LFC-1))*(TVU(LFC-1)-TV0(LFC-1))/ &
                        (TVEN+TV0(LFC-1))
                    ELSE
                        CIN1=CIN1+G*(Z00(NK)-Z00(NK-1))* &
                        ((TVU(NK)+TVU(NK-1))/(TV0(NK)+TV0(NK-1))-1.)
                    ENDIF
                ENDDO
            ENDIF
        
        ENDIF
    !----------------------------------------------------------------------
    
        LTOP1=LTOP+1
        LTOPM1=LTOP-1
    
    !***  DEFINE VARIABLES ABOVE CLOUD TOP
    
        DO NK=LTOP1,KX
            UMF(NK)=0.
            UDR(NK)=0.
            UER(NK)=0.
            QDT(NK)=0.
            QLIQ(NK)=0.
            QICE(NK)=0.
            QLQOUT(NK)=0.
            QICOUT(NK)=0.
            DETLQ(NK)=0.
            DETIC(NK)=0.
            PPTLIQ(NK)=0.
            PPTICE(NK)=0.
        
            IF(NK > LTOP1)THEN
                TU(NK)=0.
                QU(NK)=0.
                WU(NK)=0.
            ENDIF
        
            THTA0(NK)=0.
            THTAU(NK)=0.
            EMS(NK)=0.
            EMSD(NK)=0.
            TG(NK)=T0(NK)
            QG(NK)=Q0(NK)
            QLG(NK)=0.
            QIG(NK)=0.
            QRG(NK)=0.
            QSG(NK)=0.
            OMG(NK)=0.
        ENDDO
    
        OMG(KXP1)=0.
        P150=P0(KLCL)-1.50E4
    !----------------------------------------------------------------------
        DO 375 NK=1,LTOP
            EMS(NK)=DP(NK)*DXSQ/G
            EMSD(NK)=1./EMS(NK)
        
        !***  INITIALIZE SOME VARIABLES TO BE USED LATER IN THE
        !***  VERTICAL ADVECTION SCHEME
        
            EXN(NK)=(P00/P0(NK))**(0.2854*(1.-0.28*QDT(NK)))
            THTAU(NK)=TU(NK)*EXN(NK)
            EXN(NK)=(P00/P0(NK))**(0.2854*(1.-0.28*Q0(NK)))
            THTA0(NK)=T0(NK)*EXN(NK)
        
        !***  LVF IS THE LEVEL AT WHICH MOISTURE FLUX IS ESTIMATED AS THE
        !***  BASIS FOR PRECIPITATION EFFICIENCY CALCULATIONS.
        
            IF(P0(NK) > P150)LVF=NK
            DDILFRC(NK)=1./DILFRC(NK)
            OMG(NK)=0.
        375 END DO
    !----------------------------------------------------------------------
        LVF=MIN0(LVF,LET)
        USR=UMF(LVF+1)*(QU(LVF+1)+QLIQ(LVF+1)+QICE(LVF+1))
        USR=AMIN1(USR,TRPPT)
    
    !***  COMPUTE CONVECTIVE TIME SCALE(TIMEC). THE MEAN WIND AT THE LCL
    !***  AND MIDTROPOSPHERE IS USED.
    
        WSPD(KLCL)=SQRT(U0(KLCL)*U0(KLCL)+V0(KLCL)*V0(KLCL))
        WSPD(L5)=SQRT(U0(L5)*U0(L5)+V0(L5)*V0(L5))
        WSPD(LTOP)=SQRT(U0(LTOP)*U0(LTOP)+V0(LTOP)*V0(LTOP))
        VCONV=0.5*(WSPD(KLCL)+WSPD(L5))
        TIMEC=DX(I,J)/VCONV
        TADVEC=TIMEC
        TIMEC=AMAX1(1800.,TIMEC)
        TIMEC=AMIN1(3600.,TIMEC)
        IF(ISHALL == 1)TIMEC=2400.
        NIC=NINT(TIMEC/(0.5*DT2))
        TIMEC=FLOAT(NIC)*0.5*DT2
    
    !***  COMPUTE WIND SHEAR AND PRECIPITATION EFFICIENCY.
    
        IF(WSPD(LTOP) > WSPD(KLCL))THEN
            SHSIGN=1.
        ELSE
            SHSIGN=-1.
        ENDIF
    
        VWS=(U0(LTOP)-U0(KLCL))*(U0(LTOP)-U0(KLCL))+(V0(LTOP)-V0(KLCL))* &
        (V0(LTOP)-V0(KLCL))
        VWS=1.E3*SHSIGN*SQRT(VWS)/(Z00(LTOP)-Z00(LCL))
        PEF=1.591+VWS*(-.639+VWS*(9.53E-2-VWS*4.96E-3))
        PEF=AMAX1(PEF,.2)
        PEF=AMIN1(PEF,.9)
    
    !***  PRECIPITATION EFFICIENCY IS A FUNCTION OF THE HEIGHT OF CLOUD BASE
    
        CBH=(ZLCL-Z00(1))*3.281E-3
    
        IF(CBH < 3.)THEN
            RCBH=.02
        ELSE
            RCBH=0.96729352+CBH*(-.70034167+CBH*(.162179896+CBH* &
            (-1.2569798E-2+CBH*(4.2772E-4-CBH*5.44E-6))))
        ENDIF
    
        IF(CBH > 25)RCBH=2.4
        PEFCBH=1./(1.+RCBH)
        PEFCBH=AMIN1(PEFCBH,.9)
    
    !***  MEAN PRECIP EFFICIENCY IS USED TO COMPUTE RAINFALL.
    
        PEFF=.5*(PEF+PEFCBH)
        PEFF2=PEFF
    
    !*****************************************************************
    !                                                                *
    !                  COMPUTE DOWNDRAFT PROPERTIES                  *
    !                                                                *
    !*****************************************************************
    
        TDER=0.
    
        IF(ISHALL == 1)THEN
            LFS=1
            GO TO 450
        ENDIF
    
    !***  START DOWNDRAFT ABOUT 150 MB ABOVE CLOUD BASE
    
        KSTART=KPBL+1
    
        DO NK=KSTART+1,KL
            DPPP=P0(KSTART)-P0(NK)
            IF(DPPP > 150.E2)THEN
                KLFS=NK
                GO TO 405
            ENDIF
        ENDDO
    
        405 CONTINUE
    
        KLFS=MIN0(KLFS,LET-1)
        LFS=KLFS
    
    !***  IF LFS IS NOT AT LEAST 50 MB ABOVE CLOUD BASE (IMPLYING THAT THE
    !***  LEVEL OF EQUIL TEMP, LET, IS JUST ABOVE CLOUD BASE) DO NOT ALLOW
    !***  A DOWNDRAFT.
    
        IF((P0(KSTART)-P0(LFS)) < 50.E2)THEN
            TDER=0.
            GO TO 450
        ENDIF
    
        THETED(LFS)=THETEE(LFS)
        QD(LFS)=Q0(LFS)
    
    !***  CALL TPMIX2DD TO FIND WET-BULB TEMP, QV
    
        CALL TPMIX2DD(P0(LFS),THETED(LFS),TZ(LFS),QSS)
    
        THTAD(LFS)=TZ(LFS)*(P00/P0(LFS))**(0.2854*(1.-0.28*QSS))
    
    !***  TAKE A FIRST GUESS AT THE INITIAL DOWNDRAFT MASS FLUX...
    
        TVD(LFS)=TZ(LFS)*(1.+0.608*QSS)
        RDD=P0(LFS)/(R*TVD(LFS))
        A1=(1.-PEFF)*AU0
        DMF(LFS)=-A1*RDD
        DER(LFS)=DMF(LFS)
        DDR(LFS)=0.
        RHBAR=RH(LFS)*DP(LFS)
        DPTT=DP(LFS)
    
        DO ND=LFS-1,KSTART,-1
            ND1=ND+1
            DER(ND)=DER(LFS)*EMS(ND)/EMS(LFS)
            DDR(ND)=0.
            DMF(ND)=DMF(ND1)+DER(ND)
            THETED(ND)=(THETED(ND1)*DMF(ND1)+THETEE(ND)*DER(ND))/DMF(ND)
            QD(ND)=(QD(ND1)*DMF(ND1)+Q0(ND)*DER(ND))/DMF(ND)
            DPTT=DPTT+DP(ND)
            RHBAR=RHBAR+RH(ND)*DP(ND)
        ENDDO
    
        RHBAR=RHBAR/DPTT
        DMFFRC=2.*(1.-RHBAR)
        DPDD=0.
    !----------------------------------------------------------------------
    
    !***  CALCULATE MELTING EFFECT.
    !***  FIRST, COMPUTE TOTAL FROZEN PRECIPITATION GENERATED.
    
        PPTMLT=0.
    
        DO NK=KLCL,LTOP
            PPTMLT=PPTMLT+PPTICE(NK)
        ENDDO
    
    !----------------------------------------------------------------------
        IF(LC < ML)THEN
        
        !***  FOR NOW, CALCULATE MELTING EFFECT AS IF DMF=-UMF AT KLCL, I.E.,
        !***  AS IF DMFFRC=1.  OTHERWISE, FOR SMALL DMFFRC, DTMELT GETS TOO LARGE.
        
            DTMELT=RLF*PPTMLT/(CP*UMF(KLCL))
        ELSE
            DTMELT=0.
        ENDIF
    
        LDT=MIN0(LFS-1,KSTART-1)
        CALL TPMIX2DD(P0(KSTART),THETED(KSTART),TZ(KSTART),QSS)
        TZ(KSTART)=TZ(KSTART)-DTMELT
        ES=ALIQ*EXP((BLIQ*TZ(KSTART)-CLIQ)/(TZ(KSTART)-DLIQ))
        QSS=0.622*ES/(P0(KSTART)-ES)
        THETED(KSTART)=TZ(KSTART)*(1.E5/P0(KSTART))** &
        (0.2854*(1.-0.28*QSS))* &
        EXP((3374.6525/TZ(KSTART)-2.5403)*QSS*(1.+0.81*QSS))
    
        LDT=MIN0(LFS-1,KSTART-1)
    
    !----------------------------------------------------------------------
        DO 425 ND=LDT,1,-1
            DPDD=DPDD+DP(ND)
            THETED(ND)=THETED(KSTART)
            QD(ND)=QD(KSTART)
        
        !***  CALL TPMIX2DD TO FIND WET BULB TEMP, SATURATION MIXING RATIO
        
            CALL TPMIX2DD(P0(ND),THETED(ND),TZ(ND),QSS)
            QSD(ND)=QSS
        
        !***  SPECIFY RH DECREASE OF 10%/KM IN DOWNDRAFT
        
            RHH=1.-0.2/1000.*(Z00(KSTART)-Z00(ND))
        
        !***  ADJUST DOWNDRAFT TEMP, Q TO SPECIFIED RH
        
            IF(RHH < 1.)THEN
                DSSDT=(CLIQ-BLIQ*DLIQ)/((TZ(ND)-DLIQ)*(TZ(ND)-DLIQ))
                RL=XLV0-XLV1*TZ(ND)
                DTMP=RL*QSS*(1.-RHH)/(CP+RL*RHH*QSS*DSSDT)
                T1RH=TZ(ND)+DTMP
                ES=RHH*ALIQ*EXP((BLIQ*T1RH-CLIQ)/(T1RH-DLIQ))
                QSRH=0.622*ES/(P0(ND)-ES)
            
            !***  CHECK TO SEE IF MIXING RATIO AT SPECIFIED RH IS LESS THAN
            !***  ACTUAL MIXING RATIO.  IF SO, ADJUST TO GIVE ZERO EVAPORATION.
            
                IF(QSRH < QD(ND))THEN
                    QSRH=QD(ND)
                    T1RH=TZ(ND)+(QSS-QSRH)*RL/CP
                ENDIF
            
                TZ(ND)=T1RH
                QSS=QSRH
                QSD(ND)=QSS
            ENDIF
        
            TVD(ND)=TZ(ND)*(1.+0.608*QSD(ND))
        
            IF(TVD(ND) > TV0(ND) .OR. ND == 1)THEN
                LDB=ND
                GO TO 430
            ENDIF
        
        425 END DO
    !----------------------------------------------------------------------
        430 CONTINUE
    
        IF((P0(LDB)-P0(LFS)) < 50.E2)THEN   ! No Downdraft allowed!
            TDER=0.
            GO TO 450
        ENDIF
    
        TDER=0.
    
    !***  CALCULATE AN EVAPORATION RATE FOR GIVEN MASS FLUX
    
        DO ND=LDT,LDB,-1
            ND1=ND+1
            DDR(ND)=-DMF(KSTART)*DP(ND)/DPDD
            DER(ND)=0.
            DMF(ND)=DMF(ND1)+DDR(ND)
            TDER=TDER+(QSD(nd)-QD(ND))*DDR(ND)
            QD(ND)=QSD(ND)
            THTAD(ND)=TZ(ND)*(P00/P0(ND))**(0.2854*(1.-0.28*QD(ND)))
        ENDDO
    
        450 CONTINUE
    !----------------------------------------------------------------------
    
    !***  IF DOWNDRAFT DOES NOT EVAPORATE ANY WATER FOR SPECIFIED RELATIVE
    !***  HUMIDITY, NO DOWNDRAFT IS ALLOWED.
    
        IF(TDER < 1.)THEN
            PPTFLX=TRPPT
            CPR=TRPPT
            TDER=0.
            CNDTNF=0.
            UPDINC=1.
            LDB=LFS
        
            DO NDK=1,LTOP
                DMF(NDK)=0.
                DER(NDK)=0.
                DDR(NDK)=0.
                THTAD(NDK)=0.
                WD(NDK)=0.
                TZ(NDK)=0.
                QD(NDK)=0.
            ENDDO
        
            AINCM2=100.
            GO TO 475
        ENDIF
    
    !***  ADJUST DOWNDRAFT MASS FLUX SO THAT EVAPORATION RATE IN DOWNDRAFT
    !***  IS CONSISTENT WITH PRECIPITATION EFFICIENCY RELATIONSHIP.
    
    !***  DDINC IS THE FACTOR BY WHICH TO INCREASE THE FIRST-GUESS DOWNDRAFT
    !***  MOMENTUM FLUX TO SATISFY THE PRECIP EFFICIENCY RELATIONSHIP.
    !***  UPDINC IS THE FACTOR BY WHICH TO INCREASE THE UPDRAFT MASS FLUX BELOW
    !***  THE LFS TO ACCOUNT FOR TRANSFER OF MASS FROM UPDRAFT TO DOWNDRAFT.
    
        DDINC=-DMFFRC*UMF(KLCL)/DMF(KSTART)
        UPDINC=1.
    
        IF(TDER*DDINC > TRPPT)THEN
            DDINC=TRPPT/TDER
        ENDIF
    
        TDER=TDER*DDINC
    
        DO NK=LDB,LFS
            DMF(NK)=DMF(NK)*DDINC
            DER(NK)=DER(NK)*DDINC
            DDR(NK)=DDR(NK)*DDINC
        ENDDO
    
        CPR=TRPPT
        PPTFLX=TRPPT-TDER
    
    !***  ADJUST UPDRAFT MASS FLUX, MASS DETRAINMENT RATE, AND LIQUID WATER
    !***  AND DETRAINMENT RATES TO BE CONSISTENT WITH THE TRANSFER OF THE
    !***  ESTIMATE FROM THE UPDRAFT TO THE DOWNDRAFT AT THE LFS.
    
        DO NK=LC,LFS
            UMF(NK)=UMF(NK)*UPDINC
            UDR(NK)=UDR(NK)*UPDINC
            UER(NK)=UER(NK)*UPDINC
            PPTLIQ(NK)=PPTLIQ(NK)*UPDINC
            PPTICE(NK)=PPTICE(NK)*UPDINC
            DETLQ(NK)=DETLQ(NK)*UPDINC
        ENDDO
    
    !***  ZERO OUT THE ARRAYS FOR DOWNDRAFT DATA AT LEVELS ABOVE AND BELOW
    !***  THE DOWNDRAFT
    
        IF(LDB > 1)THEN
            DO NK=1,LDB-1
                DMF(NK)=0.
                DER(NK)=0.
                DDR(NK)=0.
                WD(NK)=0.
                TZ(NK)=0.
                QD(NK)=0.
                THTAD(NK)=0.
            ENDDO
        ENDIF
    
        DO NK=LFS+1,KX
            DMF(NK)=0.
            DER(NK)=0.
            DDR(NK)=0.
            WD(NK)=0.
            TZ(NK)=0.
            QD(NK)=0.
            THTAD(NK)=0.
        ENDDO
    
        DO NK=LDT+1,LFS-1
            TZ(NK)=0.
            QD(NK)=0.
            THTAD(NK)=0.
        ENDDO
    !----------------------------------------------------------------------
        475 CONTINUE
    !----------------------------------------------------------------------
    
    !***  SET LIMITS ON THE UPDRAFT AND DOWNDRAFT MASS FLUXES SO THAT THE
    !***  INFLOW INTO CONVECTIVE DRAFTS FROM A GIVEN LAYER IS NO MORE THAN
    !***  IS AVAILABLE IN THAT LAYER INITIALLY.
    
        AINCMX=1000.
        LMAX=MAX0(KLCL,LFS)
    
        DO NK=LC,LMAX
            IF((UER(NK)-DER(NK)) > 0.)AINCM1=EMS(NK)/((UER(NK)-DER(NK))* &
            TIMEC)
            AINCMX=AMIN1(AINCMX,AINCM1)
        ENDDO
    
        AINC=1.
        IF(AINCMX < AINC)AINC=AINCMX
    
    !***  SAVE THE RELEVENT VARIABLES FOR A UNIT UPDRFT AND DOWNDRFT.
    !***  THEY WILL BE ADJUSTED ITERATIVELY BY THE FACTOR AINC TO SATISFY
    !***  THE STABILIZATION CLOSURE.
    
        NCOUNT=0
        TDER2=TDER
        PPTFL2=PPTFLX
    
        DO NK=1,LTOP
            DETLQ2(NK)=DETLQ(NK)
            DETIC2(NK)=DETIC(NK)
            UDR2(NK)=UDR(NK)
            UER2(NK)=UER(NK)
            DDR2(NK)=DDR(NK)
            DER2(NK)=DER(NK)
            UMF2(NK)=UMF(NK)
            DMF2(NK)=DMF(NK)
        ENDDO
    
        FABE=1.
        STAB=0.95
        NOITR=0
        ISTOP=0
    
        IF(AINC/AINCMX > 0.999)THEN
            NCOUNT=0
            GO TO 575
        ENDIF
    !----------------------------------------------------------------------
    
    !***  SHALLOW CONVECTION EFFECTS
    
        IF(ISHALL == 1)THEN
        
        !***  FIND THE MAXIMUM tke VALUE BETWEEN LC AND KLCL
        
            TKEMAX=0.
            DO K=LC,KLCL
                NK=KX-K+1-NLEV
                TKEMAX=AMAX1(TKEMAX,Q2(I,J,NK))
            ENDDO
        
            TKEMAX=AMIN1(TKEMAX,10.)
            TKEMAX=AMAX1(TKEMAX,5.)
        
        !***  DPMIN WAS CHANGED FOR SHALLOW CONVECTION SO THAT IT IS THE
        !***  THE SAME AS FOR DEEP CONVECTION (5.E3).  sINCE THIS DOUBLES
        !***  (ROUGHLY) THE VALUE OF DPTHMX, ADD A FACTOR OF 0.5 TO THE
        !***  CALCULATION OF EVAC.
        
            EVAC=0.5*TKEMAX*0.1
            AINC=EVAC*DPTHMX*DX(I,J)*DX(I,J)/(VMFLCL*G*TIMEC)
            GO TO 575
        ENDIF
    !----------------------------------------------------------------------
    !----------------------------------------------------------------------
    
    !*****************************************************************
    !                                                                *
    !           COMPUTE PROPERTIES FOR COMPENSATIONAL SUBSIDENCE     *
    !                                                                *
    !*****************************************************************
    
        500 NCOUNT=NCOUNT+1
    
    !***  DETERMINE OMEGA VALUE NECESSARY AT TOP AND BOTTOM OF EACH LAYER
    !***  TO SATISFY MASS CONTINUITY.
    
        DTT=TIMEC
    
        DO NK=1,LTOP
            DOMGDP(NK)=-(UER(NK)-DER(NK)-UDR(NK)-DDR(NK))*EMSD(NK)
            IF(NK > 1)THEN
                OMG(NK)=OMG(NK-1)-DP(NK-1)*DOMGDP(NK-1)
                DTT1=0.75*DP(NK-1)/(ABS(OMG(NK))+1.E-10)
                DTT=AMIN1(DTT,DTT1)
            ENDIF
        ENDDO
    
        DO NK=1,LTOP
            THPA(NK)=THTA0(NK)
            QPA(NK)=Q0(NK)
            NSTEP=NINT(TIMEC/DTT+1)
            DTIME=TIMEC/FLOAT(NSTEP)
            FXM(NK)=OMG(NK)*DXSQ/G
        ENDDO
    !----------------------------------------------------------------------
    
    !***  DO AN UPSTREAM/FORWARD-IN-TIME ADVECTION OF THETA, QV
    
        DO 525 NTC=1,NSTEP
        
        !***  ASSIGN THETA AND Q VALUES AT THE TOP AND BOTTOM OF EACH LAYER BASED
        !***  SIGN OF OMEGA.
        
            DO NK=1,LTOP
                THFXIN(NK)=0.
                THFXOUT(NK)=0.
                QFXIN(NK)=0.
                QFXOUT(NK)=0.
            ENDDO
        
            DO NK=2,LTOP
                IF(OMG(NK) <= 0.)THEN
                    THFXIN(NK)=-FXM(NK)*THPA(NK-1)
                    QFXIN(NK)=-FXM(NK)*QPA(NK-1)
                    THFXOUT(NK-1)=THFXOUT(NK-1)+THFXIN(NK)
                    QFXOUT(NK-1)=QFXOUT(NK-1)+QFXIN(NK)
                ELSE
                    THFXOUT(NK)=FXM(NK)*THPA(NK)
                    QFXOUT(NK)=FXM(NK)*QPA(NK)
                    THFXIN(NK-1)=THFXIN(NK-1)+THFXOUT(NK)
                    QFXIN(NK-1)=QFXIN(NK-1)+QFXOUT(NK)
                ENDIF
            ENDDO
        
        !***  UPDATE THE THETA AND QV VALUES AT EACH LEVEL
        
            DO NK=1,LTOP
                THPA(NK)=THPA(NK)+(THFXIN(NK)+UDR(NK)*THTAU(NK)+DDR(NK)* &
                THTAD(NK)-THFXOUT(NK)-(UER(NK)-DER(NK))*THTA0(NK))* &
                DTIME*EMSD(NK)
                IF(NK >= LC .AND. NK <= KPBL)THEN
                    QPA(NK)=QPA(NK)+(QFXIN(NK)+UDR(NK)*QDT(NK)+DDR(NK)*QD(NK)- &
                    QFXOUT(NK)-UER(NK)*QUER(NK)+DER(NK)*Q0(NK))* &
                    DTIME*EMSD(NK)
                ELSE
                    QPA(NK)=QPA(NK)+(QFXIN(NK)+UDR(NK)*QDT(NK)+DDR(NK)*QD(NK)- &
                    QFXOUT(NK)-(UER(NK)-DER(NK))*Q0(NK))*DTIME*EMSD(NK)
                ENDIF
            ENDDO
        
        525 END DO
    !----------------------------------------------------------------------
        DO NK=1,LTOP
            THTAG(NK)=THPA(NK)
            QG(NK)=QPA(NK)
        ENDDO
    !----------------------------------------------------------------------
    
    !***  CHECK TO SEE IF MIXING RATIO DIPS BELOW ZERO ANYWHERE.  IF SO,
    !***  BORROW MOISTURE FROM ADJACENT LAYERS TO BRING IT BACK UP ABOVE
    !***  ZERO.
    
        DO 530 NK=1,LTOP
        
            IF(QG(NK) < 0.)THEN
            
                IF(NK == 1)THEN
                    PRINT *,'!!!!! PROBLEM WITH KF SCHEME:  '
                    PRINT *,'QG = 0 AT THE SURFACE!!!!!!!'
                    STOP 'QG'
                ENDIF
            
                NK1=NK+1
                IF(NK == LTOP)NK1=KLCL
                TMA=QG(NK1)*EMS(NK1)
                TMB=QG(NK-1)*EMS(NK-1)
                TMM=(QG(NK)-1.E-9)*EMS(NK  )
                BCOEFF=-TMM/((TMA*TMA)/TMB+TMB)
                ACOEFF=BCOEFF*TMA/TMB
                TMB=TMB*(1.-BCOEFF)
                TMA=TMA*(1.-ACOEFF)
            
                IF(NK == LTOP)THEN
                    QVDIFF=(QG(NK1)-TMA*EMSD(NK1))*100./QG(NK1)
                
                    IF(ABS(QVDIFF) > 1.)THEN
                        PRINT*,'!!!WARNING!!! CLOUD BASE WATER VAPOR CHANGES BY ', &
                        QVDIFF, &
                        '% WHEN MOISTURE IS BORROWED TO PREVENT NEGATIVE ', &
                        'VALUES IN KAIN-FRITSCH'
                    ENDIF
                
                ENDIF
            
                QG(NK)=1.E-9
                QG(NK1)=TMA*EMSD(NK1)
                QG(NK-1)=TMB*EMSD(NK-1)
            ENDIF
        
        530 END DO
    !----------------------------------------------------------------------
        TOPOMG=(UDR(LTOP)-UER(LTOP))*DP(LTOP)*EMSD(LTOP)
    
        IF(ABS(TOPOMG-OMG(LTOP)) > 1.E-3)THEN
            WRITE(99,*)'ERR:  MASS NOT BALANCED IN KF SCHEME; TOPOMG, OMG =' &
            ,TOPOMG,OMG(LTOP)
            WRITE(6,*)'I, J, NTSD =',I,J,NTSD
            WRITE(6,*)'ERR:  MASS NOT BALANCED IN KF SCHEME; TOPOMG, OMG =' &
            ,TOPOMG,OMG(LTOP)
            ISTOP=1
            GO TO 600
        ENDIF
    
    !***  CONVERT THETA TO T
    
        DO NK=1,LTOP
            EXN(NK)=(P00/P0(NK))**(0.2854*(1.-0.28*QG(NK)))
            TG(NK)=THTAG(NK)/EXN(NK)
            TVG(NK)=TG(NK)*(1.+0.608*QG(NK))
        ENDDO
    
    !***  SHALLOW
    
        IF(ISHALL == 1)THEN
            GO TO 600
        ENDIF
    
    !*******************************************************************
    !                                                                  *
    !     COMPUTE NEW CLOUD AND CHANGE IN AVAILABLE BUOYANT ENERGY.    *
    !                                                                  *
    !*******************************************************************
    
    !***  THE FOLLOWING COMPUTATIONS ARE SIMILAR TO THAT FOR UPDRAFT
    
        TMIX=0.
        QMIX=0.
    
    !***  FIND THE THERMODYNAMIC CHARACTERISTICS OF THE LAYER BY
    !***  MASS-WEIGHTING THE CHARACTERISTICS OF THE INDIVIDUAL MODEL
    !***  LAYERS.
    
        DO NK=LC,KPBL
            TMIX=TMIX+DP(NK)*TG(NK)
            QMIX=QMIX+DP(NK)*QG(NK)
        ENDDO
    
        TMIX=TMIX/DPTHMX
        QMIX=QMIX/DPTHMX
        ES=ALIQ*EXP((TMIX*BLIQ-CLIQ)/(TMIX-DLIQ))
        QSS=0.622*ES/(PMIX-ES)
    
    !***  REMOVE SUPERSATURATION FOR DIAGNOSTIC PURPOSES, IF NECESSARY
    
        IF(QMIX > QSS)THEN
            RL=XLV0-XLV1*TMIX
            CPM=CP*(1.+0.887*QMIX)
            DSSDT=QSS*(CLIQ-BLIQ*DLIQ)/((TMIX-DLIQ)*(TMIX-DLIQ))
            DQ=(QMIX-QSS)/(1.+RL*DSSDT/CPM)
            TMIX=TMIX+RL/CP*DQ
            QMIX=QMIX-DQ
            TLCL=TMIX
        ELSE
            QMIX=AMAX1(QMIX,0.)
            EMIX=QMIX*PMIX/(0.622+QMIX)
            ASTRT=1.E-3
            BINC=0.075
            A1=EMIX/ALIQ
            TP=(A1-ASTRT)/BINC
            INDLU=INT(TP)+1
            VALUE=(INDLU-1)*BINC+ASTRT
            AINTRP=(A1-VALUE)/BINC
            TLOG=AINTRP*ALU(INDLU+1)+(1-AINTRP)*ALU(INDLU)
            TDPT=(CLIQ-DLIQ*TLOG)/(BLIQ-TLOG)
            TLCL=TDPT-(.212+1.571E-3*(TDPT-T00)-4.36E-4*(TMIX-T00))*(TMIX- &
            TDPT)
            TLCL=AMIN1(TLCL,TMIX)
        ENDIF
    
        TVLCL=TLCL*(1.+0.608*QMIX)
        ZLCL=ZMIX+(TLCL-TMIX)/GDRY
    
        DO NK=LC,KL
            KLCL=NK
            IF(ZLCL <= Z00(NK))GO TO 550
        ENDDO
    
        PRINT*,'PROBLEM...LCL cannot be found after conv adjustment!!'
        STOP 'KF BAD ADJUST'
    
        550 CONTINUE
        K=KLCL-1
        DLP=(ZLCL-Z00(K))/(Z00(KLCL)-Z00(K))
    
    !***  ESTIMATE ENVIRONMENTAL TEMPERATURE AND MIXING RATIO AT THE LCL
    
        TENV=TG(K)+(TG(KLCL)-TG(K))*DLP
        QENV=QG(K)+(QG(KLCL)-QG(K))*DLP
        TVEN=TENV*(1.+0.608*QENV)
        PLCL=P0(K)+(P0(KLCL)-P0(K))*DLP
        THETEU(K)=TMIX*(1.E5/PMIX)**(0.2854*(1.-0.28*QMIX))* &
        EXP((3374.6525/TLCL-2.5403)*QMIX*(1.+0.81*QMIX))
        ES=ALIQ*EXP((TENV*BLIQ-CLIQ)/(TENV-DLIQ))
        QESE=0.622*ES/(PLCL-ES)
        THTESG(K)=TENV*(1.E5/PLCL)**(0.2854*(1.-0.28*QESE))* &
        EXP((3374.6525/TENV-2.5403)*QESE*(1.+0.81*QESE))
    !----------------------------------------------------------------------
    
    !***  COMPUTE ADJUSTED ABE(ABEG)
    
        ABEG=0.
        THTUDL=THETEU(K)
        THEU_ADJ=THTUDL
    
        DO 560 NK=K,LTOPM1
            NK1=NK+1
            THETEU(NK1)=THETEU(NK)
        
            CALL TP_CAPE(P0(NK1),THETEU(NK1),TGU(NK1),QGU(NK1))
        
            TVQU(NK1)=TGU(NK1)*(1.+0.608*QGU(NK1)-QLIQ(NK1)-QICE(NK1))
        
            IF(NK == K)THEN
                DZZ=Z00(KLCL)-ZLCL
                DILBE=((TVLCL+TVQU(NK1))/(TVEN+TVG(NK1))-1.)*DZZ
            ELSE
                DZZ=DZA(NK)
                DILBE=((TVQU(NK)+TVQU(NK1))/(TVG(NK)+TVG(NK1))-1.)*DZZ
            ENDIF
        
            IF(DILBE > 0.)ABEG=ABEG+DILBE*G
        
        !*** DILUTE BY ENTRAINMENT BY THE RATE AS ORIGINAL UPDRAFT.
        
            CALL ENVIRTHT(P0(NK1),TG(NK1),QG(NK1),THTEEG(NK1),0., &
            RL,ALIQ,BLIQ,CLIQ,DLIQ,AICE,BICE,CICE,DICE)
            THETEU(NK1)= &
            THETEU(NK1)*DDILFRC(NK1)+THTEEG(NK1)*(1.-DDILFRC(NK1))
        560 END DO
    !----------------------------------------------------------------------
    
    !***  ASSUME AT LEAST 90% OF CAPE (ABE) IS REMOVED BY CONVECTION DURING
    !***  THE PERIOD TIMEC
    
        IF(NOITR == 1)THEN
            GO TO 600
        ENDIF
    
        DABE=AMAX1(ABE-ABEG,0.1*ABE)
        FABE=ABEG/(ABE+1.E-8)
    
        IF(FABE > 1. .AND. ISHALL == 0)THEN
        !         WRITE(98,*)'UPDRAFT/DOWNDRAFT COUPLET INCREASES CAPE AT THIS'
        !         WRITE(98,*)'GRID POINT; NO CONVECTION ALLOWED!'
            GO TO 900
        ENDIF
    
        IF(NCOUNT /= 1)THEN
            DFDA=(FABE-FABEOLD)/(AINC-AINCOLD)
            IF(DFDA > 0.)THEN
                NOITR=1
                AINC=AINCOLD
                GO TO 575
            ENDIF
        ENDIF
    
        AINCOLD=AINC
        FABEOLD=FABE
    
        IF(AINC/AINCMX > 0.999 .AND. FABE > 1.05-STAB)THEN
            GO TO 600
        ENDIF
    
        IF(FABE <= 1.05-STAB .AND. FABE >= 0.95-STAB)GO TO 600
        IF(NCOUNT > 10)THEN
            GO TO 600
        ENDIF
    !----------------------------------------------------------------------
    
    !***  IF MORE THAN 10% OF THE ORIGINAL CAPE REMAINS, INCREASE THE
    !***  CONVECTIVE MASS FLUX BY THE FACTOR AINC
    
        IF(FABE == 0.)THEN
            AINC=AINC*0.5
        ELSE
            AINC=AINC*STAB*ABE/(DABE+1.E-8)
        ENDIF
    
        575 AINC=AMIN1(AINCMX,AINC)
    
    !***  IF AINC BECOMES VERY SMALL, EFFECTS OF CONVECTION
    !***  WILL BE MINIMAL SO JUST IGNORE IT.
    
        IF(AINC < 0.05)THEN
            GO TO 900
        ENDIF
    
    !     AINC=AMAX1(AINC,0.05)
        TDER=TDER2*AINC
        PPTFLX=PPTFL2*AINC
    
        DO NK=1,LTOP
            UMF(NK)=UMF2(NK)*AINC
            DMF(NK)=DMF2(NK)*AINC
            DETLQ(NK)=DETLQ2(NK)*AINC
            DETIC(NK)=DETIC2(NK)*AINC
            UDR(NK)=UDR2(NK)*AINC
            UER(NK)=UER2(NK)*AINC
            DER(NK)=DER2(NK)*AINC
            DDR(NK)=DDR2(NK)*AINC
        ENDDO
    !***
    !***  GO BACK UP FOR ANOTHER ITERATION...
    !***
        GO TO 500
        600 CONTINUE
    !----------------------------------------------------------------------
    
    !***  COMPUTE HYDROMETEOR TENDENCIES AS IS DONE FOR T, QV
    
    !***  FRC2 IS THE FRACTION OF TOTAL CONDENSATE
    !***  GENERATED THAT GOES INTO PRECIPITIATION
    
        IF(CPR > 0.)THEN
            FRC2=PPTFLX/(CPR*AINC)
        ELSE
            FRC2=0.
        ENDIF
    
        DO NK=1,LTOP
            QLPA(NK)=QL0(NK)
            QIPA(NK)=QI0(NK)
            QRPA(NK)=QR0(NK)
            QSPA(NK)=QS0(NK)
            RAINFB(NK)=PPTLIQ(NK)*AINC*FBFRC*FRC2
            SNOWFB(NK)=PPTICE(NK)*AINC*FBFRC*FRC2
        ENDDO
    
    !----------------------------------------------------------------------
        DO 625 NTC=1,NSTEP
        
        !***  ASSIGN HYDROMETEORS CONCENTRATIONS AT THE TOP AND BOTTOM OF EACH
        !***  LAYER BASED ON THE SIGN OF OMEGA
        
            DO NK=1,LTOP
                QLFXIN(NK)=0.
                QLFXOUT(NK)=0.
                QIFXIN(NK)=0.
                QIFXOUT(NK)=0.
                QRFXIN(NK)=0.
                QRFXOUT(NK)=0.
                QSFXIN(NK)=0.
                QSFXOUT(NK)=0.
            ENDDO
        
            DO NK=2,LTOP
                IF(OMG(NK) <= 0.)THEN
                    QLFXIN(NK)=-FXM(NK)*QLPA(NK-1)
                    QIFXIN(NK)=-FXM(NK)*QIPA(NK-1)
                    QRFXIN(NK)=-FXM(NK)*QRPA(NK-1)
                    QSFXIN(NK)=-FXM(NK)*QSPA(NK-1)
                    QLFXOUT(NK-1)=QLFXOUT(NK-1)+QLFXIN(NK)
                    QIFXOUT(NK-1)=QIFXOUT(NK-1)+QIFXIN(NK)
                    QRFXOUT(NK-1)=QRFXOUT(NK-1)+QRFXIN(NK)
                    QSFXOUT(NK-1)=QSFXOUT(NK-1)+QSFXIN(NK)
                ELSE
                    QLFXOUT(NK)=FXM(NK)*QLPA(NK)
                    QIFXOUT(NK)=FXM(NK)*QIPA(NK)
                    QRFXOUT(NK)=FXM(NK)*QRPA(NK)
                    QSFXOUT(NK)=FXM(NK)*QSPA(NK)
                    QLFXIN(NK-1)=QLFXIN(NK-1)+QLFXOUT(NK)
                    QIFXIN(NK-1)=QIFXIN(NK-1)+QIFXOUT(NK)
                    QRFXIN(NK-1)=QRFXIN(NK-1)+QRFXOUT(NK)
                    QSFXIN(NK-1)=QSFXIN(NK-1)+QSFXOUT(NK)
                ENDIF
            ENDDO
        !----------------------------------------------------------------------
        
        !***  UPDATE THE HYDROMETEOR CONCENTRATION VALUES AT EACH LEVEL
        
            DO 620 NK=1,LTOP
                QLPA(NK)=QLPA(NK)+(QLFXIN(NK)+DETLQ(NK)-QLFXOUT(NK))*DTIME* &
                EMSD(NK)
                QIPA(NK)=QIPA(NK)+(QIFXIN(NK)+DETIC(NK)-QIFXOUT(NK))*DTIME* &
                EMSD(NK)
            
            !***  REVERSE THE MOD THAT ALLOWS FEEDBACK OF RAIN/SNOW THAT ORIGINATES
            !***  IN DETRAINED AIR.
            
            !     QRPA(NK)=QRPA(NK)+(QRFXIN(NK)+QLQOUT(NK)*UDR(NK)-QRFXOUT(NK)
                QRPA(NK)=QRPA(NK)+(QRFXIN(NK)-QRFXOUT(NK) &
                +RAINFB(NK))*DTIME*EMSD(NK)
            
            !***  REVERSE THE MOD THAT ALLOWS FEEDBACK OF RAIN/SNOW THAT ORIGINATES
            !***  IN DETRAINED AIR.
            
            !     QSPA(NK)=QSPA(NK)+(QSFXIN(NK)+QICOUT(NK)*UDR(NK)-QSFXOUT(NK)
                QSPA(NK)=QSPA(NK)+(QSFXIN(NK)-QSFXOUT(NK) &
                +SNOWFB(NK))*DTIME*EMSD(NK)
            
            620 END DO
        625 END DO
    !----------------------------------------------------------------------
    !----------------------------------------------------------------------
        DO NK=1,LTOP
            QLG(NK)=QLPA(NK)
            QIG(NK)=QIPA(NK)
            QRG(NK)=QRPA(NK)
            QSG(NK)=QSPA(NK)
        ENDDO
    !----------------------------------------------------------------------
    
    !***  CLEAN THINGS UP, CALCULATE CONVECTIVE FEEDBACK TENDENCIES FOR
    !***  THIS GRID POINT.
    
    !***  SEND FINAL PARAMETERIZED VALUES TO OUTPUT FILES.
    
    ! 640 IF(IPRNT)THEN
    !       IF(ISTOP.EQ.1)THEN
    !         WRITE(99,*)
    !         WRITE(99,*)'At T(H), I, J =',FLOAT(NTSD)*72./3600.,I,J
    !         WRITE(99,*)'P(LC), DTP, WKL, WKLCL =',P0(LC)/100.,
    !    1                TLCL+DTLCL+DTRH-TENV,WKL,WKLCL
    !         WRITE(99,*)'TLCL, DTLCL, DTRH, TENV =',TLCL,DTLCL,
    !    1                DTRH,TENV
    !         WRITE(99,1025)KLCL,ZLCL,DTLCL,LTOP,P0(LTOP),IFLAG,
    !    1    TMIX-T00,PMIX,QMIX,ABE
    !         WRITE(99,1030)P0(LET)/100.,P0(LTOP)/100.,VMFLCL,PLCL/100.,
    !    1    WLCL,CLDHGT(LC)
    !         WRITE(99,1035)PEF,PEFCBH,LC,LET,WKL,VWS
    !         WRITE(99,*)'PRECIP EFFICIENCY =',PEFF
    !         WRITE(99,1080)LFS,LDB,LDT,TIMEC,TADVEC,NSTEP,NCOUNT,FABE,AINC
    !       ENDIF
    
    !***  IF CALCULATIONS ABOVE SHOW AN ERROR IN THE MASS BUDGET, PRINT OUT
    !***  FOR USE LATER FOR DIAGNOSTIC PURPOSES, THEN ABORT RUN.
    
    !       IF(ISHALL.EQ.0.OR.ISTOP.EQ.1)THEN
    !         WRITE(98)I,J,IYR,IMO,IDY,IHR,IMN,KL
    !         WRITE(98) P0,T0,Q0,U0,V0,W0,DP,TKE
    !         IF(ISTOP.EQ.1)THEN
    !           STOP 'KAIN-FRITSCH'
    !         ENDIF
    !       ENDIF
    !     ENDIF
    !----------------------------------------------------------------------
    
        CNDTNF=(1.-EQFRC(LFS))*(QLIQ(LFS)+QICE(LFS))*DMF(LFS)
    !***
    !***  EVALUATE MOISTURE BUDGET
    !***
        QINIT=0.
        QFNL=0.
        DPT=0.
    
        DO NK=1,LTOP
            DPT=DPT+DP(NK)
            QINIT=QINIT+Q0(NK)*EMS(NK)
            QFNL=QFNL+QG(NK)*EMS(NK)
            QFNL=QFNL+(QLG(NK)+QIG(NK)+QRG(NK)+QSG(NK))*EMS(NK)
        ENDDO
    
        QFNL=QFNL+PPTFLX*TIMEC*(1.-FBFRC)
    !     QFNL=QFNL+PPTFLX*TIMEC
    
        ERR2=(QFNL-QINIT)*100./QINIT
    
        IF(ABS(ERR2) > 0.05 .AND. ISTOP == 0)THEN
            WRITE(99,*)'!!!!!!!! MOISTURE BUDGET ERROR IN KFPARA !!!'
            WRITE(99,666)QINIT,QFNL,ERR2
            666 FORMAT(' QINIT=',E12.5,' QFNL=',E12.5,' ERR2=',E12.5)
        !       IPRNT=.TRUE.
        !       ISTOP=1
        !       GO TO 640
            STOP 'QVERR'
        ENDIF
    
        RELERR=ERR2*QINIT/(PPTFLX*TIMEC+1.E-10)
    !----------------------------------------------------------------------
    
    !***  FEEDBACK TO RESOLVABLE SCALE TENDENCIES.
    
    !***  IF THE ADVECTIVE TIME PERIOD (TADVEC) IS LESS THAN SPECIFIED MINIMUM
    !***  TIMEC, ALLOW FEEDBACK TO OCCUR ONLY DURING TADVEC.
    
        IF(TADVEC < TIMEC)NIC=NINT(TADVEC/(0.5*DT2))
        NCA(I,J)=NIC
    
        IF(ISHALL == 1)THEN
            TIMEC=2400.
            NCA(I,J)=NCLDCK
            NSHALL=NSHALL+1
        ENDIF
    
        NKFP=NKFP+1
    
    !----------------------------------------------------------------------
        DO 675 K=1,KX-NLEV
            NK=KX-K+1-NLEV
        
        !***  EVAPORATE OR SUBLIMATE ALL HYDROMETEORS SO THAT THE
        !***  FEEDBACKS ARE TEMPERATURE AND WATER VAPOR TENDENCIES.  THIS MAY
        !***  CREATE SUPERSATURATED VALUES OF TG, BUT THESE WILL BE REMOVED BY
        !***  NORMAL SUPERSATURATION-REMOVAL MECHANISMS.
        
            QLG(K)=QLG(K)+QIG(K)+QRG(K)+QSG(K)
        
            IF(ISHALL == 1)THEN
                RL=XLV0-XLV1*TG(K)
                QG(K)=QG(K)+QLG(K)
                TG(K)=TG(K)-RL*QLG(K)/CP
                QLG(K)=0.
            ! LG
            ELSE
                RL=XLV0-XLV1*TG(K)
                QG(K)=QG(K)+(QLG(K)*FBFRC)*FBFQv
                TG(K)=TG(K)-RL*((QLG(K)*FBFRC)*FBFQv)/CP
                QLG(K)=QLG(K)-(QLG(K)*FBFRC)*FBFQv
            ! LG
            ENDIF
        
            DTDT(I,J,NK)=(TG(K)-T0(K))/TIMEC
            DQDT(I,J,NK)=(QG(K)-Q0(K))/TIMEC
            DQCDT(I,J,NK)=(QLG(K)-QL0(K))/TIMEC
        
            IF(ISHALL == 0)THEN
                NCAD(I,J)=NCA(I,J)
                PSRC(I,J)=PMIX0/100.
                PCLB(I,J)=PLCL0/100.
                EMST=DPTHMX*DXSQ/G
                UMFB(I,J)=100.*VMFLCL*TIMEC*AINC/EMST
                CIN(I,J)=CIN1
            ENDIF
        
        675 END DO
    
    !--------------SAVE CLOUD TOP AND BOTTOM FOR RADIATION------------------
    
        LTOP=KL-LTOP+1
        LBOT=KL-LCL+1
        HTOP(I,J)=MIN(FLOAT(LTOP),HTOP(I,J))
        HBOT(I,J)=MAX(FLOAT(LBOT),HBOT(I,J))
        CNVTOP(I,J)=MIN(FLOAT(LTOP),CNVTOP(I,J))
        CNVBOT(I,J)=MAX(FLOAT(LBOT),CNVBOT(I,J))
    
        RAINCV(I,J)=.1*.5*DT2*PPTFLX*(1.-FBFRC)/DXSQ
        RNC=RAINCV(I,J)*NIC
        NCCNT=NCCNT+1
    900 END DO
!----------------------------------------------------------------------
!***  END OF MARCH THROUGH THE POINTS
!----------------------------------------------------------------------
!      WRITE(*)'AT NTSD =',NTSD,',NO. OF KF POINTS ACTIVATED =',
!    1             NCCNT
!----------------------------------------------------------------------
    RETURN
    END SUBROUTINE KFPARA
!**********************************************************************
    SUBROUTINE TP_CAPE(P,THES,TU,QU)

    PARAMETER(KFNT=250,KFNP=220)

    COMMON/KFLUT/ TTAB(KFNT,KFNP),QSTAB(KFNT,KFNP),THE0K(KFNP), &
    ALU(200),RDPR,RDTHK,PTOP
!----------------------------------------------------------------------
!***  SCALING PRESSURE & TT TABLE INDEX
!----------------------------------------------------------------------
    TP=(P-PTOP)*RDPR
    QQ=TP-AINT(TP)
    IPTB=INT(TP)+1

!----------------------------------------------------------------------
!***  BASE AND SCALING FACTOR FOR THE
!----------------------------------------------------------------------

!----------------------------------------------------------------------
!***  SCALING THE & TT TABLE INDEX
!----------------------------------------------------------------------
    BTH=(THE0K(IPTB+1)-THE0K(IPTB))*QQ+THE0K(IPTB)
    TTH=(THES-BTH)*RDTHK
    PP =TTH-AINT(TTH)
    ITHTB=INT(TTH)+1

    T00=TTAB(ITHTB  ,IPTB  )
    T10=TTAB(ITHTB+1,IPTB  )
    T01=TTAB(ITHTB  ,IPTB+1)
    T11=TTAB(ITHTB+1,IPTB+1)

    Q00=QSTAB(ITHTB  ,IPTB  )
    Q10=QSTAB(ITHTB+1,IPTB  )
    Q01=QSTAB(ITHTB  ,IPTB+1)
    Q11=QSTAB(ITHTB+1,IPTB+1)

!----------------------------------------------------------------------
!***  PARCEL TEMPERATURE
!----------------------------------------------------------------------

    TU=(T00+(T10-T00)*PP+(T01-T00)*QQ &
    +(T00-T10-T01+T11)*PP*QQ)

    QU=(Q00+(Q10-Q00)*PP+(Q01-Q00)*QQ &
    +(Q00-Q10-Q01+Q11)*PP*QQ)
!----------------------------------------------------------------------
    RETURN
    END SUBROUTINE TP_CAPE

